
/****
 *
 *  CImgWndPane.c
 *
 *	Image window edit methods for the SignWriter symbol editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/



#include "CImgWndPane.h"
#include <string.h>
#include <Global.h>
#include <Commands.h>
#include "CBartender.h"
#include "CCharGrid.h"
#include "CClipboard.h"
#include "CFileWndGrid.h"
#include "CImgWndClear.h"
#include "CImgWndImage.h"
#include "CImgWindow.h"
#include "CImgWndSelDrag.h"
#include "CImgWndSelect.h"
#include "CImgWndSelDrag.h"
#include "CImgWndTask.h"
#include "CImgWndToolCoord.h"
#include "CImgWndToolDraw.h"
#include "CImgWndToolEdit.h"
#include "CView.h"
#include "LongCoordinates.h"
#include "Symbols.h"
#include "SymEditConsts.h"
#include "TBUtilities.h"




extern CBartender	*gBartender;		/* Manages all menus				*/
extern CClipboard	*gClipboard;		/* Copies and Pastes data			*/
extern CBureaucrat	*gGopher;			/* First in line to get commands	*/
extern RgnHandle	gUtilRgn;			/* Utility region					*/
extern long			gSleepTime;			/* event period						*/

extern CFileWndGrid	*gFileWind;			/* File window grid					*/

extern CBitMap			*gScratchPad;
extern CursHandle		gPencilCurs;
extern CursHandle		gEraserCurs;
extern CursHandle		gDragCurs;
extern CursHandle		gSelectCurs;




/***
/* 
/*  IImgWndPane
/*
/*  Initialize image window editor
/*
 ***/

void CImgWndPane::IImgWndPane (CView *anEnclosure, CBureaucrat *aSupervisor,
							   short aWidth, short aHeight,
							   short aHEncl, short aVEncl,
							   SizingOption	aHSizing, SizingOption aVSizing,
							   short symCode, short symVar, short fillVar, short image,
							   CSymWndGrid *symWind)
{
	LongPt	frame;
	LongPt	center;
	
	// save image reference
	itsSymCode = symCode;
	itsSymVar  = symVar;
	itsFillVar = fillVar;
	itsImage   = image;
	
	// save sym window reference
	itsSymWind = symWind;
	
	// init bitmap pane
	CPane::IPane(anEnclosure, aSupervisor,
				 aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
					 
	// accept mouse input
	wantsClicks = TRUE;
	
	// default edit tool
	itsCurrentTool = toolFRAME;
	
	// init selection
	SetLongRect(&selRect, 0, 0, 0, 0);
	
	// init tool pane
	itsToolPane = NULL;
	
    // create image bitmap
    itsBitMap = new CBitMap;
	itsBitMap->IBitMap(IMAP_WIDTH, IMAP_HEIGHT, TRUE);
	
	// get existing image
	GetImage(itsSymCode, itsSymVar, itsFillVar, itsImage,
			 itsBitMap->macBitMap, &frame, &center);
			 
	// init image coords
	BitMapToPanePt(&frame, &itsFrame);
	BitMapToPanePt(&center, &itsCenter);

}



/***
/* 
/*  Dispose
/*
 ***/
 
void CImgWndPane::Dispose()
{
	if (itsBitMap != NULL) {
		itsBitMap->Dispose();
	}
	
	inherited::Dispose();
}



/***
/* 
/*  SetToolPane
/*
/*	Install tool pane associated with the image
/*
 ***/
 
void CImgWndPane::SetToolPane (CCharGrid *pane)
{
	itsToolPane = pane;
}



/***
/*  BitMapToPaneRect
/* 
/* 	Convert bitmap rectangle to pane rectangle
/*	
 ***/

void CImgWndPane::BitMapToPaneRect (Rect *mapRect, Rect *paneRect)
{
	paneRect->top  = mapRect->top * IPANE_SCALE;
	paneRect->left = mapRect->left * IPANE_SCALE;
	paneRect->bottom = mapRect->bottom * IPANE_SCALE;
	paneRect->right  = mapRect->right * IPANE_SCALE;
}



/***
/*
/*  PaneToBitMapRect
/* 
/* 	Convert pane rectangle to bitmap rectangle
/* 	
 ***/

void CImgWndPane::PaneToBitMapRect (Rect *paneRect, Rect *mapRect)
{
	mapRect->top  = paneRect->top / IPANE_SCALE;
	mapRect->left = paneRect->left / IPANE_SCALE;
	mapRect->bottom = paneRect->bottom / IPANE_SCALE;
	mapRect->right  = paneRect->right / IPANE_SCALE;
}



/***
/*  LBitMapToPaneRect
/* 
/* 	Convert long bitmap rectangle to pane rectangle
/*	
 ***/

void CImgWndPane::LBitMapToPaneRect (LongRect *mapRect, LongRect *paneRect)
{
	paneRect->top  = mapRect->top * IPANE_SCALE;
	paneRect->left = mapRect->left * IPANE_SCALE;
	paneRect->bottom = mapRect->bottom * IPANE_SCALE;
	paneRect->right  = mapRect->right * IPANE_SCALE;
}



/***
/*
/*  LPaneToBitMapRect
/* 
/* 	Convert long pane rectangle to bitmap rectangle
/* 	
 ***/

void CImgWndPane::LPaneToBitMapRect (LongRect *paneRect, LongRect *mapRect)
{
	mapRect->top  = paneRect->top / IPANE_SCALE;
	mapRect->left = paneRect->left / IPANE_SCALE;
	mapRect->bottom = paneRect->bottom / IPANE_SCALE;
	mapRect->right  = paneRect->right / IPANE_SCALE;
}



/***
/*
/*  PaneToBitMapPt
/*
/* 	Convert pane coordinate to bitmap coordinate
/*
 ***/	

void CImgWndPane::PaneToBitMapPt (LongPt *panePt, LongPt *mapPt)
{
	mapPt->h = panePt->h / IPANE_SCALE;
	mapPt->v = panePt->v / IPANE_SCALE;
}



/***
/*
/*  BitMapToPanePt
/*
/* 	Convert bitmap coordinate to pane coordinate
/*
 ***/	

void CImgWndPane::BitMapToPanePt (LongPt *mapPt, LongPt *panePt)
{
	panePt->h = mapPt->h * IPANE_SCALE;
	panePt->v = mapPt->v * IPANE_SCALE;
}



/***
/* DoCommand
/*
/*  Manage image window commands
/*
 ***/

void CImgWndPane::DoCommand (long theCommand)

{
	if (theCommand < 0) {
		switch (HiShort(-theCommand)) {
		
			case cmdEditTool:
				KillSelRect();
				KillFrame();
				KillCenter();	
				
				itsCurrentTool = LoShort(-theCommand);
				
				UpdateFrame();
				UpdateCenter();	
				break;
				
			default:
				inherited::DoCommand(theCommand);
		}
	} else {
		switch (theCommand) {
		
			case cmdCut:
				DoCopy();
				DoClear(iCUT);
				break;
				
			case cmdCopy:
				DoCopy();
				break;
				
			case cmdPaste:
				DoPaste();
				break;
				
			case cmdClear:
				DoClear(iCLEAR);
				break;
			
			case cmdRevertImage:
				DoRevertImage();
				break;
				
			case cmdInvertVert:
				DoInvertVert();
				break;
			
			case cmdInvertHorz:
				DoInvertHorz();
				break;
			
			case cmdRotate45:
				DoRotate45();
				break;
				
			case cmdRotate90:
				DoRotate90();
				break;
			
			case cmdEnlarge50:
				DoEnlarge50();
				break;
					
			default:
				inherited::DoCommand(theCommand);
				break;
		}
	}
}



/***
/* 
/*  UpdateMenus
/*
/*  Manage image window menus
/*
 ***/

void CImgWndPane::UpdateMenus()

{
	inherited::UpdateMenus();
	
	
  	// Apple menu
  	
  	gBartender->EnableCmd(cmdAbout);
  	
  	
  	// file management
  	
  	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
	
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	gBartender->EnableCmd(cmdRevert);
 		

	// Image editing
	
	if (gGopher == this) {
	
		gBartender->EnableCmd(cmdRevertImage);

		gBartender->EnableCmd(cmdInvertVert);
		gBartender->EnableCmd(cmdInvertHorz);
		gBartender->EnableCmd(cmdRotate45);
		gBartender->EnableCmd(cmdRotate90);
		gBartender->EnableCmd(cmdEnlarge50);

		if (!EmptyLongRect(&this->selRect)) {
			gBartender->EnableCmd(cmdCut);
			gBartender->EnableCmd(cmdCopy);
			gBartender->EnableCmd(cmdClear);
		}
		
		if (gClipboard->DataSize('PICT') > 0) {
			gBartender->EnableCmd(cmdPaste);
		}
	}
}



/***
/* 
/*  DoClick (override default)
/*
/*  Handle mouse click in edit pane
/*
 ***/

void CImgWndPane::DoClick (Point hitPt, short modifierKeys, long when)

{
	CImgWndTask		*theEditTask;
	LongRect		tselRect;
	LongRect		tframe;
	Rect			bounds;
	Boolean			undo = TRUE;
	Boolean			notifiable = TRUE;
	LongPt			trackStart;

	tframe = this->frame;
	tselRect = this->selRect;
	
	LongToQDRect(&tframe, &bounds);
	QDToLongPt(hitPt, &trackStart);

	if (!PtInRect(hitPt, &bounds)) {
		return;
	}
	
	if (itsCurrentTool != toolSELECT) {	/* Editing zaps current selection */
		KillSelRect();
	}
	
	switch (itsCurrentTool) {
	
		case toolSELECT:			/* We could be selecting or dragging	*/
		
			notifiable = FALSE;
			if (PtInLongRect( &trackStart, &tselRect)) {
									/* Dragging the current selection		*/
									
				if ((itsLastTask == NULL) || !member(itsLastTask, CSelDrag)) {
									/* This is a brand new drag, NOT the	*/
									/*   continuation of a previous one		*/
						
					itsLastTask = new CSelDrag;
					((CSelDrag*)itsLastTask)->ISelDrag(this, itsBitMap, iDRAG);
					
					notifiable = TRUE;
				}
					
				theEditTask = (CImgWndTask*) itsLastTask;
				
			} else {				/* Make a selection rectangle			*/
				undo = FALSE;
				KillSelRect();
				
				theEditTask = new CSelect;
				((CSelect*)theEditTask)->ISelect(this, itsBitMap);
				
				gSleepTime = 0;		/* Cursor shape needs to change within	*/
									/*   selection, so we must force an		*/
									/*   idle so that the mouse region will	*/
									/*   be updated							*/
			}
			break;
		
		case toolDRAG:
		
			theEditTask = new CToolDrag;
			((CToolDrag*)theEditTask)->IToolDrag(this, itsBitMap);
			break;
			
		case toolPENCIL:
		
			theEditTask = new CToolPencil;
			((CToolPencil*)theEditTask)->IToolPencil(this, itsBitMap);
			break;
			
		case toolERASER:
		
			theEditTask = new CToolEraser;
			((CToolEraser*)theEditTask)->IToolEraser(this, itsBitMap);
			break;
	
		case toolDRAW:
			if (!KeyIsDown(OPTKEY_CODE)) {
			
				theEditTask = new CToolLine;
				((CToolLine*)theEditTask)->IToolDraw(this, itsBitMap);
				
			} else {
			
				theEditTask = new CToolCircle;
				((CToolCircle*)theEditTask)->IToolDraw(this, itsBitMap);
			}
			break;
			
		case toolFRAME:
	
			undo = FALSE;
			theEditTask = new CToolFrame;
			((CToolFrame*)theEditTask)->IToolFrame(this, itsBitMap);
			break;
		
		case toolCENTER:
	
			undo = FALSE;
			theEditTask = new CToolCenter;
			((CToolCenter*)theEditTask)->IToolCenter(this, itsBitMap);
			break;
	}
	
									/* Mouse down drawing within the	*/
									/*   bounds of the edit window		*/
	ForceNextPrepare();
	Prepare();
	  
	TrackMouse(theEditTask, &trackStart, &tframe);
	
	if (!undo) {					/* Operation is not undoable so we	*/
		theEditTask->Dispose();		/*   don't need the task anymore	*/
	} else {
		if (notifiable) {			/* Make sure we don't re-notify in	*/
									/*   the case of a resumed drag		*/
			itsSupervisor->Notify(theEditTask);
		}
		itsLastTask = theEditTask;	/* Save task for later reference	*/
	}
}
	
	
	
/***
/* 
/*  AdjustCursor (override default)
/*
/*  Change cursor shape to denote mouse position.
/*
 ***/

void CImgWndPane::AdjustCursor (Point where, RgnHandle mouseRgn)

{
	LongRect	tselRect;
	LongRect	tframe;	
	Rect		mouseRect;
	LongPt		mouseLoc;

	tframe = this->frame;
	tselRect = this->selRect;
	
	WindToFrame(where, &mouseLoc);
	if (!PtInLongRect( &mouseLoc, &tframe)) {
	
		// Mouse outside drawing area
		SetCursor(&arrow);
		FrameToGlobalR(&tframe, &mouseRect);
		RectRgn(gUtilRgn, &mouseRect);
		DiffRgn(mouseRgn, gUtilRgn, mouseRgn);
		return;
	}
	
	switch (itsCurrentTool) {				/* Cursor shape depends on tool */
	
		case toolSELECT:
			if (PtInLongRect( &mouseLoc, &tselRect)) {
				SetCursor(*gDragCurs);
			} else {
				SetCursor(*gSelectCurs);
			}
			break;
		
		case toolPENCIL:
			SetCursor(*gPencilCurs);
			break;
			
		case toolERASER:
			SetCursor(*gEraserCurs);
			break;
		
		case toolDRAG:
			SetCursor(*gDragCurs);
			break;
	
		case toolFRAME:
		case toolCENTER:
			SetCursor(*gSelectCurs);
			break;
}
	
	if (!EmptyLongRect(&tselRect)) {		/* Cursor changes shape inside		*/
											/*   selection rectangle			*/
		FrameToGlobalR(&tselRect, &mouseRect);
		RectRgn(gUtilRgn, &mouseRect);
		
		if (PtInLongRect( &mouseLoc, &tselRect)) {
			SectRgn(mouseRgn, gUtilRgn, mouseRgn);
		} else {
			DiffRgn(mouseRgn, gUtilRgn, mouseRgn);
		}
	}
}



/***
/* 
/*  Dawdle
/*
/*  Animate the selection during Idle time 
/*
 ***/

void CImgWndPane::Dawdle (long	*maxSleep)

{
	Pattern		thePat;
	long		ticks;
	Rect		rect;
	
	if (!EmptyLongRect(&this->selRect)) {	  /* Make sure selection exists */
		LongToQDRect(&this->selRect, &rect);
		Prepare();
		
		PenNormal();				/* Draw rectangle with one pattern		*/
		GetIndPattern(thePat, PAT_SEL_ANIM, 1);
		PenPat(thePat);
		FrameRect(&rect);
		
		Delay(6, &ticks);			/* Wait a while							*/
			
									/* Redraw with a pattern offset from	*/
									/*   the first. This gives the			*/
									/*   illusion of marching ants.			*/
									
		GetIndPattern(thePat, PAT_SEL_ANIM, 2);
		PenPat(thePat);
		FrameRect(&rect);
		*maxSleep = 5;
	}
}



/***
/* 
/*  KillSelRect
/*
/*  Zap selection rectangle and finalize the last task
/*
 ***/

void CImgWndPane::KillSelRect()

{
	LongRect rect;
	
	if (itsLastTask != NULL) {		/* Killing selection means we are about	*/
		itsLastTask->Do();			/*   to do something new. Let the last	*/
		Notify(NULL);				/*   task finalize its actions.			*/
		itsLastTask = NULL;			
	}
	
	LPaneToBitMapRect(&this->selRect, &rect);	 // Redraw to erase selection frame
	Update(itsBitMap->macBitMap, &rect);

	SetLongRect(&this->selRect, 0, 0, 0, 0);	 // Set empty selection
}



/***
/* 
/*  KillFrame
/*
/*  Zap image frame display
/*
 ***/

void CImgWndPane::KillFrame()

{
	LongPt		pt;	
	LongRect	rect;
	short		saveTool;
	
	
	if (itsCurrentTool == toolFRAME) {
		saveTool = itsCurrentTool;
		itsCurrentTool = toolNONE;	// temp value
		
		PaneToBitMapPt(&itsFrame, &pt);
		
		SetLongRect(&rect, pt.h, 0, pt.h + 1, pt.v + 1);
		Update(itsBitMap->macBitMap, &rect);

		SetLongRect(&rect, 0, pt.v, pt.h + 1, pt.v + 1);
		Update(itsBitMap->macBitMap, &rect);
		
		itsCurrentTool = saveTool;
	}
}



/***
/* 
/*  KillCenter
/*
/*  Zap image center point display
/*
 ***/

void CImgWndPane::KillCenter()

{	
	LongPt		pt;	
	LongRect	rect;
	short		saveTool;
		
	if (itsCurrentTool == toolCENTER) {
		saveTool = itsCurrentTool;
		itsCurrentTool = toolNONE;	// temp value
		
		PaneToBitMapPt(&itsCenter, &pt);
		
		SetLongRect(&rect, pt.h, 0, pt.h + 1, IMAP_HEIGHT);
		Update(itsBitMap->macBitMap, &rect);

		SetLongRect(&rect, 0, pt.v, IMAP_WIDTH, pt.v + 1);
		Update(itsBitMap->macBitMap, &rect);
		
		itsCurrentTool = saveTool;
	}
}
	
	

/***
/* 
/*  Deactivate
/*
/*  Zap selection when deactivated 
/*
 ***/

void CImgWndPane::Deactivate()

{
	inherited::Deactivate();
	
	if (ReallyVisible()) {			/* Don't bother to kill selection if	*/
		KillSelRect();				/*   Pane is not visible				*/
	}
}
	


/***
/* 
/*  DoCopy
/*
/*  Copy selection to the clipboard. Not undoable.
/*
 ***/

void CImgWndPane::DoCopy()
{
	PicHandle		thePic;
	LongRect		rect;
	Rect			qdRect;
	
	Prepare();
	itsBitMap->BeginDrawing();
	
	LPaneToBitMapRect(&this->selRect, &rect);
	FrameToQDR(&rect, &qdRect);
									/* Open picture to store selection	*/
	thePic = OpenPicture(&qdRect);
									/* Draw selection on top of itself	*/
	itsBitMap->CopyFrom(&rect, &rect, NULL);
	ClosePicture();
	
	itsBitMap->EndDrawing();
									/* Copy picture to the clipboard	*/
	gClipboard->EmptyScrap();
	gClipboard->PutData('PICT', (Handle)thePic);
    KillPicture(thePic);
}
	


/***
/* 
/*  DoPaste
/*
/*  Paste picture from the clipboard.
/*
 ***/

void CImgWndPane::DoPaste()

{
	PicHandle		thePic;
	LongPt			anyPt;
	Rect			pFrame;
	LongRect		rect;
	
	if (gClipboard->GetData('PICT', (Handle*) &thePic)) {	
		
										/* Set tool to selection rectangle	*/
										/*   so pasted image can be dragged	*/	
		itsToolPane->Prepare();
		itsToolPane->ChangeSelection(toolSELECT);		
		DoCommand(-( (((long)cmdEditTool) << 16) + toolSELECT));

										/* Paste is treated as a drag in	*/
		itsLastTask = new CSelDrag;		/*   progress						*/
		((CSelDrag*)itsLastTask)->ISelDrag(this, itsBitMap, iPASTE);
		
										/* Place pasted picture at the		*/
										/* same position as the cut			*/
		pFrame = (**thePic).picFrame;		
		QDToLongRect(&pFrame, &rect);		

										/* Save image under the paste		*/
		CopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
					&pFrame, &pFrame, srcCopy, NULL);
			
		itsBitMap->BeginDrawing();		/* Draw the pasted image			*/
		DrawPicture(thePic, &pFrame);
		itsBitMap->EndDrawing();
							 			/* Update pane display				*/
		Update(itsBitMap->macBitMap, &rect);
		
										/* Simulate a selection drag...		*/
		LBitMapToPaneRect(&rect, &this->selRect);
		anyPt = topLeftL(this->selRect);		
										
										/* Set up the drag					*/
		((CSelDrag*)itsLastTask)->BeginTracking(&anyPt);
		
										/* Restore image under paste		*/
		CopyBits(gScratchPad->macBitMap, itsBitMap->macBitMap,
					&pFrame, &pFrame, srcCopy, NULL);
		
										/* Track drag						*/
		((CSelDrag*)itsLastTask)->EndTracking(&anyPt, &anyPt, &anyPt);
		
		itsSupervisor->Notify(itsLastTask);
        KillPicture(thePic);     
	}
}	



/***
/* 
/*  DoClear
/*
/*  Erase the current selection
/*
 ***/

void CImgWndPane::DoClear (short taskIndex)

{
	CImgWndClear  *theClearTask;
										
	theClearTask = new CImgWndClear;
	theClearTask->IImgWndClear(taskIndex, this, itsBitMap);
	theClearTask->Do();
	
	itsSupervisor->Notify(theClearTask);
	itsLastTask = theClearTask;
}



/***
/* 
/*  DoRevertImage
/*
/*  Revert to original image
/*
 ***/

void CImgWndPane::DoRevertImage (void)

{
	LongRect	bounds;
	LongPt		frame;
	LongPt		center;

	PositionDialog('ALRT', ALRTrevertImage);
	
	if (CautionAlert(ALRTrevertImage, NULL) == OK) {
	
		Prepare();
		itsBitMap->GetBounds(&bounds);	

		// erase current image		
		itsBitMap->BeginDrawing();
		LEraseRect(&bounds);
		itsBitMap->EndDrawing();
		
		// reload original image
		GetImage(itsSymCode, itsSymVar, itsFillVar, itsImage,
				 itsBitMap->macBitMap, &frame, &center);
		
		BitMapToPanePt(&frame, &this->itsFrame);
		BitMapToPanePt(&center, &this->itsCenter);
	
		// update display
		Update(itsBitMap->macBitMap, &bounds);		 	
	}		
	 
	// revert not undoable
	itsSupervisor->Notify(NULL);
}



/***
/* 
/*  Close
/*
/*  Check image frame and center point.
/*  If invalid, post alert and bail;
/*	otherwise, save current image in
/*  symbol database. 
/*
 ***/
 
Boolean CImgWndPane::Close()
{
	LongPt	pixPt;
	LongPt	frame;
	LongPt	center;
	short	x, y;
	short	result;
	short	frameX;
	short	frameY;
	Boolean found  = FALSE;
	
		
	// get frame coordinates
	
	PaneToBitMapPt(&itsFrame, &pixPt);
	frameX = pixPt.h;
	frameY = pixPt.v;
		
	
	// scan for bits to right of frame	
	
	for (x = frameX; x < IMAP_WIDTH; x++) {
		for (y = 0; y < IMAP_HEIGHT; y++) {	
			
			SetLongPt(&pixPt, x, y); 
			if (itsBitMap->PixelIsBlack(&pixPt)) {
						
				found = TRUE;
				goto exit;
			}
		}
	} 
	
	
	// scan for bits below frame
	
	for (y = frameY; y < IMAP_HEIGHT; y++) {
		for (x = 0; x < frameX; x++) {
		
			SetLongPt(&pixPt, x, y); 
			if (itsBitMap->PixelIsBlack(&pixPt)) {
						
				found = TRUE;
				goto exit;
			}
		}
	}
			
	exit:
		
	// fail if bits outside frame
	
	if (found) {
		PositionDialog('ALRT', ALRTbadFrame);	
		result = Alert(ALRTbadFrame, NULL);	
		
		return FALSE;
	}
	
	
	// fail if center outside frame
	
	if ((itsCenter.h > itsFrame.h) || (itsCenter.v > itsFrame.v)) {
		PositionDialog('ALRT', ALRTbadCenter);
		result = Alert(ALRTbadCenter, NULL);	
		
		return FALSE;
	}
	
	
	// save existing image

	PaneToBitMapPt(&itsFrame, &frame);
	PaneToBitMapPt(&itsCenter, &center);
	
	SetImage(itsSymCode, itsSymVar, itsFillVar, itsImage,
		     itsBitMap->macBitMap, &frame, &center);
		     
		     
	// update file window display
	
	if (itsSymVar  == 1 && itsFillVar == 1 && itsImage == 1)
		gFileWind->UpdateItem(itsSymCode);
	
	
	return TRUE;
}



/***
/* 
/*  DoInvertVert
/*
/*  Invert image about horizontal axis
/*
 ***/

void CImgWndPane::DoInvertVert (void)
{
	CInvertVert	*theInvert;
						
	KillSelRect();
					
	theInvert = new CInvertVert;
	theInvert->IImageTask(iINVERT, this, itsBitMap);
	theInvert->Do();
	
	itsSupervisor->Notify(theInvert);
	itsLastTask = theInvert;
}



/***
/* 
/*  DoInvertHorz
/*
/*  Invert image about vertical axis
/*
 ***/

void CImgWndPane::DoInvertHorz (void)
{
	CInvertHorz	*theInvert;
										
	KillSelRect();
	
	theInvert = new CInvertHorz;
	theInvert->IImageTask(iINVERT, this, itsBitMap);
	theInvert->Do();
	
	itsSupervisor->Notify(theInvert);
	itsLastTask = theInvert;
}



/***
/* 
/*  DoRotate45
/*
/*  Rotate image 45 degrees counterclockwise
/*
 ***/

void CImgWndPane::DoRotate45 (void)
{
	CRotate45	*theRotate;
										
	KillSelRect();

	theRotate = new CRotate45;
	theRotate->IImageTask(iROTATE, this, itsBitMap);
	theRotate->Do();
	
	itsSupervisor->Notify(theRotate);
	itsLastTask = theRotate;
}



/***
/* 
/*  DoRotate90
/*
/*  Rotate image 90 degrees counterclockwise
/*
 ***/

void CImgWndPane::DoRotate90 (void)
{
	CRotate90	*theRotate;
										
	KillSelRect();
	
	theRotate = new CRotate90;
	theRotate->IImageTask(iROTATE, this, itsBitMap);
	theRotate->Do();
	
	itsSupervisor->Notify(theRotate);
	itsLastTask = theRotate;
}



/***
/* 
/*  DoEnlarge50
/*
/*  Enlarge image 50 percent
/*
 ***/

void CImgWndPane::DoEnlarge50 (void)
{
	CEnlarge50	*theEnlarge;
	
	KillSelRect();
									
	theEnlarge = new CEnlarge50;
	theEnlarge->IImageTask(iENLARGE, this, itsBitMap);
	theEnlarge->Do();
	
	itsSupervisor->Notify(theEnlarge);
	itsLastTask = theEnlarge;
}



/***
/* 
/*  Draw (override default)
/*
/*  Draw the image bitmap
/*
 ***/

void CImgWndPane::Draw (Rect *area)

{
	LongRect	bounds;
	
	itsBitMap->GetBounds(&bounds);	
	Update(itsBitMap->macBitMap, &bounds);
}



/***
/* 
/*  Update
/*
/*  Update the image pane
/*
 ***/

void CImgWndPane::Update (BitMap *map, LongRect *area)

{
	int			x, y;
	LongRect	paneArea;
	
	Prepare();
	
	// copy image to pane
	LBitMapToPaneRect(area, &paneArea);
	LCopyBits(map, &thePort->portBits, area, &paneArea, srcCopy, NULL);
	
	// update pane grid
	PenNormal();	
	for (x = 0; x <= IPANE_WIDTH; x += IMAP_GRID * IPANE_SCALE) {
		for (y = 0; y <= IPANE_HEIGHT; y += IMAP_GRID * IPANE_SCALE) {
			MoveTo(x, y);
			LineTo(x, y);	
		}
	}
	
	// update image coordinates
	UpdateFrame();
	UpdateCenter();
	
	// update sym window display
	itsSymWind->UpdateItem(itsSymVar, itsFillVar, itsImage, map, area);
}

	

/***
/* 
/*  UpdateFrame
/*
/*  Update image frame display
/*
 ***/

void CImgWndPane::UpdateFrame()

{
	if (itsCurrentTool == toolFRAME) {
	
		Prepare();
		PenMode(patCopy);
		PenPat(gray);

		LMoveTo(itsFrame.h, 0);
		LLineTo(itsFrame.h, itsFrame.v);
		LLineTo(0, itsFrame.v);
	}
}



/***
/* 
/*  UpdateCenter
/*
/*  Update image center point display
/*
 ***/

void CImgWndPane::UpdateCenter()

{
	if (itsCurrentTool == toolCENTER) {
	
		Prepare();
		PenMode(patCopy);
		PenPat(gray);
		
		LMoveTo(itsCenter.h, 0);
		LLineTo(itsCenter.h, IPANE_HEIGHT);
		
		LMoveTo(0, itsCenter.v);
		LLineTo(IPANE_WIDTH, itsCenter.v);
	}
}