/******************************************************************************
 CImgWndTask.c

							The ImgWndTask Class
		
	SUPERCLASS = CMouseTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TCLUtilities.h>
#include "CImgWndTask.h"

/*** Global Variables ***/

extern CBitMap		*gScratchPad;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IImgWndTask

		Initialize an ImgWndTask object
 ******************************************************************************/

void CImgWndTask::IImgWndTask (short aNameIndex, CImgWndPane *aPane, CBitMap *aBitMap)

{
	inherited::IMouseTask(aNameIndex);

	itsPane = aPane;
	itsBitMap = aBitMap;
	itsUndoBits = NULL;
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of an ImgWndTask object
 ******************************************************************************/

void CImgWndTask::Dispose()

{
	if (itsUndoBits != NULL) {
		itsUndoBits->Dispose();
	}
	if (itsPane && (itsPane->itsLastTask == this))
	{
		itsPane->itsLastTask = NULL;
	}
	
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 Undo {OVERRIDE}

		Undo an action performed by an edit task
 ******************************************************************************/

void CImgWndTask::Undo()

{
	LongRect	bounds;
	
	if (itsUndoBits != NULL) {
	
		itsPane->Prepare();
		itsUndoBits->GetBounds(&bounds);
		
		LCopyBits(itsBitMap->macBitMap, gScratchPad->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		LCopyBits(itsUndoBits->macBitMap, itsBitMap->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		LCopyBits(gScratchPad->macBitMap, itsUndoBits->macBitMap,
					&bounds, &bounds, srcCopy, NULL);
					
		itsPane->Update(itsBitMap->macBitMap, &bounds);
	}
}


/******************************************************************************
 SaveForUndo

		Save an area of the bitmap so a change can be undone
 ******************************************************************************/

void CImgWndTask::SaveForUndo (LongRect *undoRect)

{
	Boolean		savedAlloc;
									/* Put changed area of painting into	*/
									/*   an undo bitmap						*/
	itsUndoBits = new CBitMap;
	
	savedAlloc = SetAllocation(kAllocCanFail);
	itsUndoBits->IBitMap(undoRect->right - undoRect->left,
						 undoRect->bottom - undoRect->top, FALSE);
	SetAllocation(savedAlloc);
	
	FailNIL(itsUndoBits);

	itsUndoBits->SetBoundsOrigin(undoRect->left, undoRect->top);
			
	LCopyBits(itsBitMap->macBitMap, itsUndoBits->macBitMap,
			  undoRect, undoRect, srcCopy, NULL);
}