/******************************************************************************
 CImgWndToolCoord.c

					The ToolFrame & ToolCenter Classes
		
	SUPERCLASS = CImgWndTask
	
    Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <TBUtilities.h>
#include "CImgWndToolCoord.h"
#include "SymEditConsts.h"



/******************************************************************************
 IToolFrame

		Initialize a ToolFrame object. There is no task name,
		since framing is not undoable.
 ******************************************************************************/

void	CToolFrame::IToolFrame(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(NOTHING, aPane, aBitMap);
}


/******************************************************************************
 ToggleFrame
 
 		Fast frame draw/erase.
 ******************************************************************************/
 
void CToolFrame::ToggleFrame()
{
	LongPt	framePt = itsPane->itsFrame;
	 
	itsPane->Prepare();
	
	PenMode(patXor);
	PenPat(gray);
	
	LMoveTo(framePt.h, 0);
	LLineTo(framePt.h, framePt.v);
	LLineTo(0, framePt.v);
}


/******************************************************************************
 BeginTracking

		Erase current frame and draw Xor version (for less flickering).
 ******************************************************************************/

void CToolFrame::BeginTracking (LongPt *startPt)
{
	// erase old frame
	itsPane->KillFrame();
	
	// redraw Xor version
	ToggleFrame();
}


/******************************************************************************
 KeepTracking
 ******************************************************************************/

void	CToolFrame::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 	currMapPt;
	LongPt	 	prevMapPt;
	
	// Convert to bitmap coordinates
	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(prevPt, &prevMapPt);
	
	if (!EqualLongPt(&currMapPt, &prevMapPt)) {		
	
		// calc new pos								
		SubLongPt(&prevMapPt, &currMapPt);
		itsPane->PaneToBitMapPt(&itsPane->itsFrame, &prevMapPt);
		AddLongPt(&prevMapPt, &currMapPt);
		
		// bounds check
		if (currMapPt.h < 1) currMapPt.h = 1;		
		if (currMapPt.v < 1) currMapPt.v = 1;
			
		if (currMapPt.h >= IMAP_WIDTH) currMapPt.h = IMAP_WIDTH;		
		if (currMapPt.v >= IMAP_HEIGHT) currMapPt.v = IMAP_HEIGHT;
		
		// erase old frame
		ToggleFrame();
		
		// update frame pos
		itsPane->BitMapToPanePt(&currMapPt, &itsPane->itsFrame);
		
		// draw new frame;
		ToggleFrame();	
	}
}


/******************************************************************************
 EndTracking
 ******************************************************************************/

void	CToolFrame::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	KeepTracking(currPt, prevPt, startPt);
}




/******************************************************************************
 IToolCenter

		Initialize a ToolCenter object. There is no task name,
		since changing the center point is not undoable.
 ******************************************************************************/

void	CToolCenter::IToolCenter(
	CImgWndPane	*aPane,
	CBitMap		*aBitMap)
{
	CImgWndTask::IImgWndTask(NOTHING, aPane, aBitMap);
}


/******************************************************************************
 ToggleCenter
 
 		Fast center point draw/erase.
 ******************************************************************************/
 
void CToolCenter::ToggleCenter()
{
	LongPt	centerPt = itsPane->itsCenter;
	 
	itsPane->Prepare();
	
	PenMode(patXor);
	PenPat(gray);
	
	LMoveTo(centerPt.h, 0);
	LLineTo(centerPt.h, IPANE_HEIGHT);
	
	LMoveTo(0, centerPt.v);
	LLineTo(IPANE_WIDTH, centerPt.v);
}


/******************************************************************************
 BeginTracking

		Erase current center point and draw Xor version (for less flickering).
 ******************************************************************************/

void CToolCenter::BeginTracking (LongPt *startPt)
{
	// erase old center point
	itsPane->KillCenter();
	
	// redraw Xor version
	ToggleCenter();
}


/******************************************************************************
 KeepTracking
 ******************************************************************************/

void	CToolCenter::KeepTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	LongPt	 	currMapPt;
	LongPt	 	prevMapPt;
	
	// Convert to bitmap coordinates
	itsPane->PaneToBitMapPt(currPt, &currMapPt);
	itsPane->PaneToBitMapPt(prevPt, &prevMapPt);
	
	if (!EqualLongPt(&currMapPt, &prevMapPt)) {		
	
		// calc new pos								
		SubLongPt(&prevMapPt, &currMapPt);
		itsPane->PaneToBitMapPt(&itsPane->itsCenter, &prevMapPt);
		AddLongPt(&prevMapPt, &currMapPt);
		
		// bounds check
		if (currMapPt.h < 1) currMapPt.h = 1;		
		if (currMapPt.v < 1) currMapPt.v = 1;
		
		if (currMapPt.h >= IMAP_WIDTH) currMapPt.h = IMAP_WIDTH - 1;		
		if (currMapPt.v >= IMAP_HEIGHT) currMapPt.v = IMAP_HEIGHT - 1;
		
		// erase old center point
		ToggleCenter();
		
		// update center point pos
		itsPane->BitMapToPanePt(&currMapPt, &itsPane->itsCenter);
		
		// draw new center point;
		ToggleCenter();	
	}
}


/******************************************************************************
 EndTracking
 ******************************************************************************/

void	CToolCenter::EndTracking(
	LongPt		*currPt,
	LongPt		*prevPt,
	LongPt		*startPt)
{
	KeepTracking(currPt, prevPt, startPt);
}

