/****
 * CSymWindow.c
 *
 *	Symbol window methods for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 ****/

#include "Global.h"
#include "Commands.h"
#include "Controls.h"
#include "CApplication.h"
#include "CBartender.h"
#include "CButton.h"
#include "CDesktop.h"
#include "CPaneBorder.h"
#include "CSymWindow.h"
#include "CSymWndDialog.h"
#include "CSymWndLabel.h"
#include "CSymWndGrid.h"
#include "CSymWndBox.h"
#include "CSymWndCtrl.h"
#include "CWindow.h"
#include "SymEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"
#include <constants.h>
#include <BDC.h>





/*** Global Variables ***/


extern	CBartender	*gBartender;	/* The menu handling object 			    */
extern	CDesktop	*gDesktop;		/* The enclosure for all windows 			*/
extern	CBureaucrat	*gGopher;		/* The current boss in the chain of command */

extern	short		 gImgWinds;     /* Image windows open			 			*/

extern  void		*gClipRec;		/* Clipboard 								*/
extern short	 	 gClipItem;




/***
 * ISymWindow
 *
 *  Initialize a new instance of a symbol window.
 *
 ***/
 
void CSymWindow::ISymWindow (CDirectorOwner *aSupervisor)

{
	itsGridPane = NULL;
	itsBoxPane  = NULL;
	itsControlPane = NULL;
	
	inherited::IDirector(aSupervisor);
}



/***
 * Dispose
 *
 *
 ***/

void CSymWindow::Dispose()

{
	inherited::Dispose();
}



/***
 * AlertLoseVars
 *
 *  Notify user that the specified
 *  operation will delete some of the
 *  currently defined symbol variations.
 *
 ***/
 
short CSymWindow::AlertLoseVars()

{
	PositionDialog('ALRT', ALRTloseVars);	
	return CautionAlert(ALRTloseVars, NULL);
}	
			
			
					
/***
 * DoCommand
 *
 *
 ***/

void CSymWindow::DoCommand (long theCommand)

{
	short  newVal, curVal;
		
	switch (theCommand) {
			
			
		case cmdSelectCol1:
		case cmdSelectCol2:
		case cmdSelectCol3:
		case cmdSelectCol4:
		case cmdSelectCol5:
		case cmdSelectCol6:

			itsGridPane->SelectColumn(theCommand - cmdSelectCol);	
			break;
		
			
		case cmdSelectAll:
		
			itsGridPane->SelectAll();
			break;
			
			
		case cmdCut:
		
			itsGridPane->DoCopy();
			itsGridPane->DoClear(iCUT);
			break;
			
				
		case cmdCopy:
		
			itsGridPane->DoCopy();
			break;
			
			
		case cmdPaste:
		
			itsGridPane->DoPaste();
			break;
			
			
		case cmdClear:
		
			itsGridPane->DoClear(iCLEAR);
			break;
		
			
		case cmdSymTypeFace:
		case cmdSymTypeHand:
		case cmdSymTypeMvmt:
		case cmdSymTypeShol:
		case cmdSymTypeCont:
		case cmdSymTypeOthr:
		
			SetSymType(itsGridPane->itsSymCode, 
					   (SymbolType) theCommand - cmdSymTypeFace);
					   
			itsBoxPane->Update();
			break;


		case cmdSymVars1:
		case cmdSymVars2:
		case cmdSymVars3:
		case cmdSymVars4:
		case cmdSymVars5:
		case cmdSymVars6:
		
			newVal = theCommand - cmdSymVars;
			GetSymVars(itsGridPane->itsSymCode, &curVal);
			
			if (newVal < curVal) {				

				if (AlertLoseVars() != OK)
					break;	
				
				if (itsGridPane->itsSymVar > newVal) {
				
					itsGridPane->itsSymVar = newVal;
					itsGridPane->ChangeSelection(SELnone);				
					itsGridPane->Refresh();	
				}
			}
			   			
   			SetSymVars(itsGridPane->itsSymCode, newVal);
   			
			itsControlPane->Update(cmdNull);
			break;	
			

		case cmdFillVars1:
		case cmdFillVars2:
		case cmdFillVars3:
		case cmdFillVars4:
		case cmdFillVars5:
		case cmdFillVars6:

			newVal = theCommand - cmdFillVars;
			GetFillVars(itsGridPane->itsSymCode, itsGridPane->itsSymVar, &curVal);
			
			if (newVal < curVal) {

				if (AlertLoseVars() != OK)
					break;		
							
				itsGridPane->ChangeSelection(SELnone);		
				itsGridPane->Refresh();	
			}
			
			SetFillVars(itsGridPane->itsSymCode, itsGridPane->itsSymVar, newVal);
			
			if (newVal > curVal)
				itsGridPane->Update(itsGridPane->itsSymVar);	
			break;	
		
		
		case cmdFillAttrs:
		
			DoFillAttrs();
			break;
		
			
		case cmdNext:
		case cmdPrevious:
		
			itsControlPane->Update(theCommand);
			itsGridPane->Update(itsControlPane->itsSetting);
			break;
					
		default:
		
			inherited::DoCommand(theCommand);
			break;
	}
}



/***
 * UpdateMenus
 *
 *
***/

 void CSymWindow::UpdateMenus (void)

 {
 	short	select;
 	short	image, junk;
 	
 	// do inherited
  	inherited::UpdateMenus();
  	
  	
  	// Apple menu
  	
  	gBartender->EnableCmd(cmdAbout);
 	
 	
  	// file management
  	
  	gBartender->DisableCmd(cmdNew);
	gBartender->DisableCmd(cmdOpen);
	gBartender->DisableCmd(cmdPrint);
	gBartender->DisableCmd(cmdPageSetup);
	
 	gBartender->EnableCmd(cmdClose);
 	gBartender->EnableCmd(cmdSave);
 	gBartender->EnableCmd(cmdSaveAs);
 	gBartender->EnableCmd(cmdRevert);	


	if (gGopher == this) {
	
		// symbol editing

		gBartender->EnableCmd(cmdSelectAll);
			
		select = itsGridPane->GetSelection();
		
		if (select != SELnone) {
	
		 	gBartender->EnableCmd(cmdCut);
		 	gBartender->EnableCmd(cmdCopy);
		 	gBartender->EnableCmd(cmdClear);
		 	
		 	
		 	// Paste command
		 	
		 	if (gClipRec != NULL) {
		 	
		 		if (select > 0)								// cell selected
		 			gBartender->EnableCmd(cmdPaste);

				else if (select != SELall) {				// column selected
								
					if (gClipItem < 0)
						gBartender->EnableCmd(cmdPaste);
				
				} else {									// all selected
				
					if (gClipItem == SELall)
						gBartender->EnableCmd(cmdPaste);
				}
			}
			
			
			// Fill Attributes command
			
			if (select > 0)	{							// top cell selected
		 	
		 		itsGridPane->ItemToSymImage(select, &junk, &image);
				if (image == 1)	
					gBartender->EnableCmd(cmdFillAttrs); 
			}	 
 		 	
		 	else if (select != SELall)					// column selected
				gBartender->EnableCmd(cmdFillAttrs); 		 
 		}
	 	
	 		
	 	// symbol attributes
	 	
	 	gBartender->EnableCmd(cmdSymType);
	 	gBartender->EnableCmd(cmdSymTypeFace);
	 	gBartender->EnableCmd(cmdSymTypeHand);
	 	gBartender->EnableCmd(cmdSymTypeMvmt);
	 	gBartender->EnableCmd(cmdSymTypeShol);
	 	gBartender->EnableCmd(cmdSymTypeCont);
	 	gBartender->EnableCmd(cmdSymTypeOthr);
	
	 	gBartender->EnableCmd(cmdSymVars);
	 	gBartender->EnableCmd(cmdSymVars1);
	 	gBartender->EnableCmd(cmdSymVars2);
	 	gBartender->EnableCmd(cmdSymVars3);
	 	gBartender->EnableCmd(cmdSymVars4);
	 	gBartender->EnableCmd(cmdSymVars5);
	 	gBartender->EnableCmd(cmdSymVars6);
	
	 	gBartender->EnableCmd(cmdFillVars);
	 	gBartender->EnableCmd(cmdFillVars1);
	 	gBartender->EnableCmd(cmdFillVars2);
	 	gBartender->EnableCmd(cmdFillVars3);
	 	gBartender->EnableCmd(cmdFillVars4);
	 	gBartender->EnableCmd(cmdFillVars5);
	 	gBartender->EnableCmd(cmdFillVars6);
	 	
	}
}



/***
 * DoFillAttrs
 *
 *	Display a dialog box allowing the user
 *	to specify the symbol attributes of the
 *  current fill variation.  The dialog box
 *  varies according to the current symbol type.
 *
 ***/

void CSymWindow::DoFillAttrs (void)

{
	CSymWndDialog   *dialog;
	SymbolType  	symType;
	ImageClass		imgClass;
	short			fillVar;
	short			rots;
	short			flops;
	long			junk;
	long			cmd = cmdNull;
	
	
	// get selected column
	
	fillVar = itsGridPane->GetSelection();
	if (fillVar < 0) fillVar = -fillVar;	// normalize column selection
				
				
	// perform appropriate dialog
	
	GetSymType(itsGridPane->itsSymCode, &symType);

	switch (symType) {
	
		case hand:
			dialog = new CHandDialog;
			((CHandDialog*) dialog)->IHandDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((CHandDialog*) dialog)->Dispose();	
			break;
			
			
		case movement:
			dialog = new CMvmtDialog;
			((CMvmtDialog*) dialog)->IMvmtDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((CMvmtDialog*) dialog)->Dispose();
			break;
			
		
		case face:
			dialog = new CFaceDialog;
			((CFaceDialog*) dialog)->IFaceDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((CFaceDialog*) dialog)->Dispose();
			break;
			
			
		case shoulder:
			dialog = new CSholDialog;
			((CSholDialog*) dialog)->ISholDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((CSholDialog*) dialog)->Dispose();
			break;
			
			
		case contact:
			dialog = new CContDialog;
			((CContDialog*) dialog)->IContDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((CContDialog*) dialog)->Dispose();
			break;
			
			
		case other:
			dialog = new COthrDialog;
			((COthrDialog*) dialog)->IOthrDialog(this);
			
			cmd = dialog->GetNewAttrs(&imgClass, &rots, &flops);
			((COthrDialog*) dialog)->Dispose();
			break;
			
			
		default:
			break;	
	}	


	// update attribute values	
		
	if (cmd == cmdOK) {
		
		SetClass(itsGridPane->itsSymCode, itsGridPane->itsSymVar, fillVar, imgClass);
		SetFlops(itsGridPane->itsSymCode, itsGridPane->itsSymVar, fillVar, flops);
		SetRotations(itsGridPane->itsSymCode, itsGridPane->itsSymVar, fillVar, rots);
		 
		 
		// update sym display  	
		
		itsGridPane->selection = SELnone;	
		itsGridPane->Refresh();	
	}	
}		
		
					
			
/***
 * NewWindow
 *
 *	Build a new instance of a symbol window and display
 *	the data specified by the argument.
 *
 ***/
	
void CSymWindow::NewWindow (short symCode, CFileWndGrid *fileWind)

{
	CSymWndGrid		*theGridPane;
	CPaneBorder		*theGridBorder;
	CSymWndLabel	*theLabelPane;
	CSymWndBox		*theBoxPane;
	CPaneBorder		*theBoxBorder;
	CSymWndCtrl		*theControlPane;
	CButton			*thePrevButton;
	CButton			*theNextButton;
	
	Rect			wLoc;
	static short	wCount = 0;
	
	Str15			winTitle = "\pSymbol ";
	Str15			numStr;

	Str15			nextTitle = "\p>>";
	Str15			prevTitle = "\p<<";

	
	   /**
		**	If empty symbol selected, initialize sym data structure. 
		**
		**/
		
	if (SymEmpty(symCode))
		ISymbol(symCode);
	
	
	   /**
		**	Get the (staggered) window location.
		**
		**/
		
	SetRect(&wLoc, SWND_LEFT, SWND_TOP, 
				   SWND_LEFT + SWND_WIDTH, SWND_TOP + SWND_HEIGHT);
				   
	OffsetRect(&wLoc, wCount * HORIZ_WOFFSET, wCount * VERT_WOFFSET);
	wCount = (wCount + 1) % MAX_WOFFSETS;
	
	
	   /**
		**	Make the (invisible) window.
		**
		**/
		
	itsWindow = new CWindow;
	itsWindow->INewWindow(&wLoc, FALSE, 4, FALSE, TRUE, gDesktop, this);
	
	
		/**
		 **  Set the window title to "Symbol X"
		 **
		 **/
		 
	NumToString(symCode, numStr);
	ConcatPStrings(winTitle, numStr);
	itsWindow->SetTitle(winTitle);
	
	
		/**
		 **	theLabelPane controls the list of grid labels
		 ** that appear across the top of the symbol window.
		 ** It is used to identify the fill variations that
		 ** appear in the symbol image grid below.
		 **
		 **/
		 
	theLabelPane = new CSymWndLabel;
    theLabelPane->ISymWndLabel(itsWindow, this,
    						   SWND_WIDTH, SLABL_HEIGHT,
    						   0, 0, 
    						   sizFIXEDLEFT, sizFIXEDTOP);
    						   
    theLabelPane->SetWantsClicks(TRUE);
    						
	
		/**
		 **	theBoxPane controls the box at the bottom left
		 ** corner of the symbol window.  It is used to 
		 ** display the type of the currently displayed
		 ** symbol.
		 **
		 **/
		 
	theBoxPane = new CSymWndBox;
    theBoxPane->ISymWndBox(itsWindow, this,
    						SBOX_WIDTH, SBOT_HEIGHT,
    						SBOX_BASE, SWND_HEIGHT - SBOT_HEIGHT, 
    						sizFIXEDLEFT, sizFIXEDBOTTOM,
    						symCode);
    						
    itsBoxPane = theBoxPane;
	
	
		/**
		 **	theBoxBorder is used to draw the righthand
		 ** border on the box pane.
		 **
		 **/
		 
	theBoxBorder = new CPaneBorder;
	theBoxBorder->IPaneBorder(kBorderRight);
	
	theBoxPane->SetBorder(theBoxBorder);

	
		/**
		 **	theNextButton and thePrevButton create the
		 ** buttons that appear at the bottom of the symbol 
		 ** window.  They are used to switch between the 
		 ** symbol variations of the current symbol.  (The
		 ** buttons are managed by the control pane.)
		 **
		 **/
		 
	thePrevButton = new CButton;
	theNextButton = new CButton;
	
	thePrevButton->INewButton(SBUT_WIDTH, SBUT_HEIGHT,
	    					  SPBT_BASE, SWND_HEIGHT - SBOT_HEIGHT + 2,
	    					  prevTitle, TRUE, pushButProc,
							  itsWindow, this);
							  
	theNextButton->INewButton(SBUT_WIDTH, SBUT_HEIGHT,
	    					  SNBT_BASE, SWND_HEIGHT - SBOT_HEIGHT + 2,
	    					  nextTitle, TRUE, pushButProc,
							  itsWindow, this);
											
	thePrevButton->SetClickCmd(cmdPrevious);
 	theNextButton->SetClickCmd(cmdNext);
 	
 	
		/**
		 **	theControlPane controls the buttons at the bottom of
		 ** the symbol window and the text message that appears between
		 ** them.  It is used to indicate the current symbol variation
		 ** displayed in the symbol window.
		 **
		 **/
		 
	theControlPane = new CSymWndCtrl;
    theControlPane->ISymWndCtrl(itsWindow, this,
    							SMSG_WIDTH, SBOT_HEIGHT,
    						    SMSG_BASE, SWND_HEIGHT - SBOT_HEIGHT, 
    						    sizFIXEDRIGHT, sizFIXEDBOTTOM,
    						    thePrevButton, theNextButton,
    						    symCode);
    						    
    itsControlPane = theControlPane;
	
	
		/**
		 **	theGridPane controls the grid array at the center
		 ** of the symbol window.  It is used to display the 
		 ** symbol images that constitute the selected symbol.
		 **
		 **/

	theGridPane = new CSymWndGrid;
    theGridPane->ISymWndGrid(itsWindow, this,
							 SWND_WIDTH, SWND_HEIGHT - (SLABL_HEIGHT + SBOT_HEIGHT),
							 0, SLABL_HEIGHT + 1, 
    						 sizFIXEDLEFT, sizFIXEDTOP,
    						 symCode);
    						 
    itsGridPane = theGridPane;
	
	
		/**
		 **	theGridBorder is used to draw top and bottom border
		 ** lines on the grid pane.
		 **
		 **/
		 
	theGridBorder = new CPaneBorder;
	theGridBorder->IPaneBorder(kBorderTop + kBorderBottom);
	
	theGridPane->SetBorder(theGridBorder);
	
	
		/**
		 **
		 ** The document class fields all sym window
		 ** commands.
		 ** 
		 **/
		 
	this->BecomeGopher(TRUE);
	
	
		/**
		 **	Link the new sym window to its file window.
		 **
		 **/
		 
	 itsFileWind = fileWind;
	 itsFileWind->SetSymWindow(symCode, this);
	 
	 
		/**
		 **	Send the window a Select() message to make
		 **	it the active window.
		 **
		 **/
	
	itsWindow->Select();
}



/***
 * Close (override)
 *
 *	Unlink from file window before closing
 *
 ***/
 
Boolean	CSymWindow::Close (Boolean quitting)

{
	itsFileWind->SetSymWindow(itsGridPane->itsSymCode, NULL);
	
	return inherited::Close(quitting);
}
	
