/****
 * CSymWndCtrl.c
 *
 *	Symbol window control methods for the SignWriter symbol editor.
 *
 *  Copyright  1995 The Center for Sutton Movement Writing.  All rights reserved.
 *
 *
 ****/


#include "CSymWndCtrl.h"
#include "CButton.h"
#include "CControl.h"
#include "Commands.h"
#include "Symbols.h"
#include "SymEditConsts.h"
#include <BDC.h>



void CSymWndCtrl::ISymWndCtrl(CView *anEnclosure, CBureaucrat *aSupervisor,
							  short aWidth, short aHeight,
							  short aHEncl, short aVEncl,
							  SizingOption aHSizing, SizingOption aVSizing,
							  CButton *thePrevButton, CButton *theNextButton,
							  short symCode)
{	
	short	symVars;
			
	itsSymCode = symCode;
	itsSetting = 1;
	
	GetSymVars(symCode, &symVars);
	
	itsPrevButton = thePrevButton;
	itsNextButton = theNextButton;
	
	itsPrevButton->Deactivate();
	if (symVars == 1) itsNextButton->Deactivate();

	inherited::IPane(anEnclosure, aSupervisor, aWidth, aHeight,
					 aHEncl, aVEncl, aHSizing, aVSizing);
}



/***
 * Draw
 *
 *	Draw the current setting of the sym window control.
 *
 ***/

void CSymWndCtrl::Draw(Rect *area)

{
	DrawSetting();
}



/***
 * DrawSetting
 *
 *	Draw the text string that shows 
 *  the current control setting.
 *
 ***/

void CSymWndCtrl::DrawSetting(void)

{
	Str27   msgStr = "Variation X of Y";
	Rect	rect;
	short	numVars;
	
	SetRect(&rect, 0, 0, SMSG_WIDTH, SBOT_HEIGHT); 
	EraseRect(&rect);
	
	msgStr[10] = '1' + itsSetting - 1;
	
	GetSymVars(itsSymCode, &numVars);
	msgStr[15] = '1' + numVars - 1;
			
	TextFont(1); TextFace(1); TextSize(10);
	MoveTo(0, SBOT_HEIGHT - 4);
	DrawString(CtoPstr(msgStr));
}



/***
 * Update
 *
 *	Update the setting of the sym window control.
 *
 ***/

void CSymWndCtrl::Update(long theCommand)

{
	short numVars;
	
	GetSymVars(itsSymCode, &numVars);
	
	
	/* update current setting */
	if ((theCommand == cmdNext) && (itsSetting < numVars)  && (itsSetting < SGRID_VARS))
		itsSetting++;
	else if ((theCommand == cmdPrevious) && (itsSetting > 1))
		itsSetting--;
	else if ((theCommand == cmdNull) && (itsSetting > numVars))
		itsSetting = numVars;
	else if (theCommand != cmdNull)
		return;
		
		
	/* update button states */
	if (itsSetting == 1)
		itsPrevButton->Deactivate();
	else
		itsPrevButton->Activate();
		
	if (itsSetting == numVars)
		itsNextButton->Deactivate();
	else
		itsNextButton->Activate();
		
		
	/*update text message */
	Prepare();
	DrawSetting();
}



/***
 * Dispose
 *
 *	Pass the buck.
 *
 ***/

void CSymWndCtrl::Dispose(void)

{
	inherited::Dispose();
}


