
/******************************************************************************
 *
 *  CSymWndTask.c
 *
 *	Symbol window task methods for the SignWriter symbol editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ******************************************************************************/
 


#include "CSymWndTask.h"
#include "CSymWndGrid.h"



/*** Global Variables ***/

extern void		*gClipRec;			/* Clipboard */
extern void		*gScratchRec;

extern short	 gClipItem;
extern short	 gScratchItem;




/********************************  Clear  **********************************/


/***
 * ISymWndClear
 *
 ***/
 
void CSymWndClear::ISymWndClear (short nameIndex, CSymWndGrid *pane, short select)

{
	CTask::ITask(nameIndex);
	
	itsGridPane = pane;
	itsSelect = select;
}	
	


/***
 * Do (override)
 *
 *	Move current selection to the global scratch record.
 *  Duplicate it, clear the resulting copy, and
 *  update the selection with the cleared copy.
 *
 ***/

void CSymWndClear::Do()

{
	void  *rec;
	
	if (gScratchRec != NULL)
		itsGridPane->DisposRec(gScratchItem, gScratchRec);
	
	itsGridPane->GetRec(itsSelect, &gScratchRec);
	gScratchItem = itsSelect;
	
	itsGridPane->CopyRec(itsSelect, gScratchRec, &rec);
	itsGridPane->ClearRec(itsSelect, &rec);
	itsGridPane->SetRec(itsSelect, rec);
					
	itsGridPane->Update(itsGridPane->itsSymVar);	
	
}
	


/***
 * Undo (override)
 *
 *	Exchange the cleared selection
 *  with the global scratch record.
 *  Re-highlight the selection.
 *
 ***/

void CSymWndClear::Undo()

{
	void  *rec;
	
	itsGridPane->GetRec(itsSelect, &rec);
	itsGridPane->SetRec(itsSelect, gScratchRec);
	gScratchRec = rec;
					
	itsGridPane->Update(itsGridPane->itsSymVar);	
	itsGridPane->ChangeSelection(itsSelect);	

}
	


/***
 * Redo (override)
 *
 *	Re-exchange the global scratch
 *  record with the cleared selection.
 *
 ***/

void CSymWndClear::Redo()

{
	void  *rec;
	
	itsGridPane->GetRec(itsSelect, &rec);
	itsGridPane->SetRec(itsSelect, gScratchRec);
	gScratchRec = rec;
					
	itsGridPane->Update(itsGridPane->itsSymVar);
		
}




/********************************  Paste  **********************************/


/***
 * ISymWndPaste
 *
 ***/
 
void CSymWndPaste::ISymWndPaste (CSymWndGrid *pane, short select)

{
	CTask::ITask(iPASTE);
	
	itsGridPane = pane;
	itsSelect = select;
	
}	


	
/***
 * Promote
 *
 *	Update the specified selection to conform
 *  to the clipboard item (e.g. cell -> column;
 *  cell -> all; column -> all).
 *
 ***/

void CSymWndPaste::Promote (short clipItem, short *select)

{
	short fill, junk;
	
	if (clipItem == SELall)
		*select = SELall;
	
	else if ((clipItem < 0) && (*select > 0)) {
		itsGridPane->ItemToSymImage(*select, &fill, &junk);
		*select = -fill;
	}
		
}



/***
 * Do (override)
 *
 *	Move target selection to the global scratch record.
 *  Duplicate the clipboard item and update the target
 *  selection with the resulting copy.
 *
 ***/

void CSymWndPaste::Do()

{
	void  *rec;
	
	if (gScratchRec != NULL)
		itsGridPane->DisposRec(gScratchItem, gScratchRec);
	
	gScratchItem = itsSelect;
	Promote(gClipItem, &gScratchItem);
	
	itsGridPane->GetRec(gScratchItem, &gScratchRec);

	itsGridPane->CopyRec(gClipItem, gClipRec, &rec);
	itsGridPane->SetRec(gScratchItem, rec);
					
	itsGridPane->Update(itsGridPane->itsSymVar);	
	
}
	


/***
 * Undo (override)
 *
 *	Exchange the pasted selection
 *  with the global scratch record.
 *  Re-highlight the original selection.
 *
 ***/

void CSymWndPaste::Undo()

{
	void  *rec;
	
	itsGridPane->GetRec(gScratchItem, &rec);
	itsGridPane->SetRec(gScratchItem, gScratchRec);
	gScratchRec = rec;
					
	itsGridPane->Update(itsGridPane->itsSymVar);	
	itsGridPane->ChangeSelection(itsSelect);	

}
	


/***
 * Redo (override)
 *
 *	Re-exchange the global scratch
 *  record with the pasted selection.
 *
 ***/

void CSymWndPaste::Redo()

{
	void  *rec;
	
	itsGridPane->GetRec(gScratchItem, &rec);
	itsGridPane->SetRec(gScratchItem, gScratchRec);
	gScratchRec = rec;
					
	itsGridPane->Update(itsGridPane->itsSymVar);	

}
