/******************************************************************************
 CAboutBox.c

							The AboutBox Class
		
	SUPERCLASS = CDirector
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <OSChecks.h>
#include <CDesktop.h>
#include <CDecorator.h>
#include <CWindow.h>
#include <CPicture.h>
#include "CAboutBox.h"
#include <CApplication.h> 

/*** Global Variables ***/

extern CDesktop		*gDesktop;			/* The visible Desktop				*/
extern CDecorator	*gDecorator;		/* Decorator for arranging windows	*/

/*** Class Constants ***/

#define		WIND_ABOUT		2000

typedef struct {
	SignedByte	h;
	SignedByte	v;
} BytePoint;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IAboutBox

		Initialize an AboutBox object
 ******************************************************************************/

void	CAboutBox::IAboutBox(
    CApplication        *itsSupervisor) 
{
	TRY
	{
		CDirector::IDirector(itsSupervisor);
		
		itsWindow = new(CWindow);
		itsWindow->IWindow(WIND_ABOUT, TRUE, gDesktop, this);
		
		gDecorator->CenterWindow(itsWindow);
		
		blackboard = GetPicture(2000);		/* Load in resources				*/
		flatEraser = GetPicture(2001);
		rightEraser = GetPicture(2002);
		leftEraser = GetPicture(2003);
		wink = GetPicture(2004);
		acHVof = GetResource('HVof', 1);
		ghdHVof = GetResource('HVof', 2);
	
		Display();
	}
	CATCH
	{
		Dispose();
	}
	ENDTRY;
}


/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of AboutBox by releasing all its resources
 ******************************************************************************/

void	CAboutBox::Dispose()
{
	ForgetResource( blackboard);
	ForgetResource( flatEraser);
	ForgetResource( rightEraser);
	ForgetResource( leftEraser);
	ForgetResource( wink);
	ForgetResource( acHVof);
	ForgetResource( ghdHVof);
	
	inherited::Dispose();
}


/******************************************************************************
 Display

		Display contents of About Box. There's a lot a ugly code here and
		hard-wired values.
 ******************************************************************************/

		/*
		 * Kill time for the specified duration, in ticks.
		 *   Exit early if the mouse is clicked or a key is pressed.
		 */
		 
		static Boolean		MouseOrKey(
			long		duration)
		{
			long		startTicks;
			KeyMap		macKeyMap;
			
			startTicks = TickCount();
			
			while ((TickCount() - startTicks) < duration) {
				GetKeys( macKeyMap);
                                
                macKeyMap[1] &= 0xFFFFFFFD; /* Mask out Caps Lock */
                macKeyMap[3] &= 0xFFFFFF7F; /* Mask out Reset key */
                
				if ( Button()  ||
					 (macKeyMap[0] != 0)  ||
					 (macKeyMap[1] != 0)  ||
					 (macKeyMap[2] != 0)  ||
					 (macKeyMap[3] != 0) ) {
						
					return(TRUE);
				}
			}
			
			return(FALSE);
		}
			
void	CAboutBox::Display()
{
	Rect				fBlackboard;
	Rect				fFlatEraser;
	Rect				fRightEraser;
	Rect				fLeftEraser;
	Rect				fWink;
	Rect				eraseClip;
	register Rect		*r;
	Rect				theRect;
	register short		i;
	short				n;
	Handle				theHVof;
	register BytePoint	**off;
	RGBColor			chalkColor;
	
	itsWindow->Select();
	itsWindow->Prepare();
	SetOrigin(-12, -12);
	
	fBlackboard = (**blackboard).picFrame;
	fFlatEraser = (**flatEraser).picFrame;
	OffsetRect(&fFlatEraser, 250, 142);
	fRightEraser = (**rightEraser).picFrame;
	fLeftEraser = (**leftEraser).picFrame;
	fWink = (**wink).picFrame;
	OffsetRect(&fWink, 51, 111);
	
	r = &theRect;
	
	DrawPicture(blackboard, &fBlackboard);
	DrawPicture(flatEraser, &fFlatEraser);
	SetRect(&eraseClip, 72, 32, 288, 108);
	
	if (ColorQDIsPresent()) {
		chalkColor.red = 65281;
		chalkColor.green = 65281;
		chalkColor.blue = 52225;
		RGBForeColor(&chalkColor);
		RGBBackColor(&chalkColor);
	} else {
		ForeColor(whiteColor);
	}

	while (TRUE) {
		if (MouseOrKey(20L)) return;
		
		SetRect(r, 80, 40, 82, 42);
		n = (short) GetHandleSize(acHVof) / sizeof(BytePoint);
		off = (BytePoint**)(acHVof);

		for (i = 0; i < n; i++) {
			OffsetRect(r, (*off)[i].h, (*off)[i].v);
			PaintRect(r);
			if (MouseOrKey(2L)) return;
		}
		
		if (MouseOrKey(60L)) return;
		ClipRect(&fFlatEraser);
		DrawPicture(blackboard, &fBlackboard);
		if (MouseOrKey(30L)) return;
		
		ClipRect(&fBlackboard);
		OffsetRect(&fRightEraser, 60 - fRightEraser.left,
								  12 - fRightEraser.top);
		DrawPicture(rightEraser, &fRightEraser);
		
		for (i = 0; i < 65; i++) {
			OffsetRect(&fRightEraser, 3, 1);
			DrawPicture(rightEraser, &fRightEraser);
			if (MouseOrKey(1L)) return;
		}
		
		for (i = 0; i < 9; i++) {
			OffsetRect(&fRightEraser, 0, 3);
			DrawPicture(rightEraser, &fRightEraser);
			if (MouseOrKey(1L)) return;
		}
		
		fLeftEraser = fRightEraser;
		DrawPicture(leftEraser, &fLeftEraser);
		for (i = 0; i < 65; i++) {
			OffsetRect(&fLeftEraser, -3, -1);
			DrawPicture(leftEraser, &fLeftEraser);
			if (MouseOrKey(1L)) return;
		}
		
		ClipRect(&fLeftEraser);
		DrawPicture(blackboard, &fBlackboard);
		if (MouseOrKey(30L)) return;
		ClipRect(&fBlackboard);
		DrawPicture(flatEraser, &fFlatEraser);
		if (MouseOrKey(30L)) return;
		
		n = (short) GetHandleSize(ghdHVof) / sizeof(BytePoint);
		off = (BytePoint**)(ghdHVof);

		SetRect(r, 90, 35, 92, 37);
		for (i = 0; i < n; i++) {
			OffsetRect(r, (*off)[i].h, (*off)[i].v);
			PaintRect(r);
			if (MouseOrKey(2L)) return;
		}
		
		if (MouseOrKey(30L)) return;
		ClipRect(&fWink);
		DrawPicture(wink, &fWink);
		if (MouseOrKey(20L)) return;
		DrawPicture(blackboard, &fBlackboard);
		if (MouseOrKey(120L)) return;
		ClipRect(&eraseClip);
		DrawPicture(blackboard, &fBlackboard);
	}
}
