/******************************************************************************
 CCaption.c

							The Caption Class
		
	SUPERCLASS = CEditText
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include <Global.h>
#include <CWindow.h>
#include "CCaption.h"

static pascal Boolean	TEClickLoop(void);


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 ICaption

		Initialize a Caption object. The Caption will use the font
		characteristics of its enclosure's port.
 ******************************************************************************/

void	CCaption::ICaption(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHEncl,
	short			aVEncl)
{
	FontInfo		macFontInfo;
	short			aHeight;
	
										/* Make initial size of Caption as	*/
										/*   wide as the widest character	*/
										/*   and a tall as a single line	*/
	GetFontInfo(&macFontInfo);
	aHeight = macFontInfo.ascent + macFontInfo.descent + macFontInfo.leading;
	
	CEditText::IEditText(anEnclosure, aSupervisor,
					 macFontInfo.widMax, aHeight, aHEncl, aVEncl,
					 sizFIXEDSTICKY, sizFIXEDSTICKY, aWidth);
	
	SetClikLoop(TEClickLoop, macTE);	/* Use dummy click loop to disable	*/
										/*   autoscrolling implemented in	*/
										/*   an EditText pane				*/
	SetWholeLines(FALSE);				/* Caption handles vertical sizing	*/
										/*   and lines in a special manner	*/
	firstCall = TRUE;					/* Kludge to prevent infinite loop	*/
}	

/******************************************************************************
 DoKeyDown

 ******************************************************************************/

void CCaption::DoKeyDown ( char  theChar,
       Byte  keyCode,
       EventRecord *macEvent)
{
	Prepare();
	TEKey(theChar, macTE);

  	FitToText();
}

/******************************************************************************
 FitToText

		Adjust the size of the frame to fit the text
 ******************************************************************************/

void	CCaption::FitToText()
{
	register short		i, pos;
	Point				loc;
	short				hMax = MININT;
	short				vMax;
	Rect				delta;
	TEPtr				t;
		
	Prepare();
	
	vMax = (**macTE).nLines;
	
	if (vMax > 0) {						/* Find the widest line of text		*/
		for (i = 1; i <= vMax; i++) {
		
			pos = (**macTE).lineStarts[i] - 1;
			loc = TEGetPoint( pos, macTE);
			hMax = Max( loc.h, hMax);
		}
	} else {
		vMax = 1;						/* Must have at least one line		*/
	}

										/* Special case for last character	*/
	loc = TEGetPoint((**macTE).teLength, macTE);
	hMax = Max(loc.h + 16, hMax);
	
	t = *macTE;
										/* TE line count is not correct if	*/
										/*   last char is a carriage return	*/
                                        
    if (t->teLength > 0) 
    
        if ( *(*t->hText + t->teLength - 1) == CARRIAGE_RETURN ) {
            vMax += 1;
        }
	
	vMax *= t->lineHeight;				/* Convert lines to pixels			*/
	
	delta.left = delta.top = 0;			/* Make Caption frame same size as	*/
	delta.right = hMax - frame.right;	/*   the text						*/
	delta.bottom = vMax - frame.bottom - frame.top;
	
	ChangeSize(&delta, TRUE);
										/* Force redraw of entire window	*/
	GetWindow()->Update();
}
	

/******************************************************************************
 DrawInPort

		Draw the Caption in the specified GrafPort
 ******************************************************************************/

void	CCaption::DrawInPort(
	GrafPtr		inPort)
{
	Rect		r;
	Point		saveOrigin;
	
	SetPort(inPort);					/* Set up port and coordinates		*/
	saveOrigin = topLeft(inPort->portRect);
	SetOrigin(-hEncl, -vEncl);
	
	(**macTE).inPort = inPort;			/* TE record has reference to port	*/
	
	r = (**macTE).viewRect;				/* Force drawing of text			*/
	TEUpdate(&r, macTE);
	
	(**macTE).inPort = macPort;			/* Restore port in TE record		*/
	
	SetOrigin(saveOrigin.h, saveOrigin.v);
	ForceNextPrepare();
}
	

/******************************************************************************
 SetFontStyle {OVERRIDE}

		Specify the style, such as bold or italic, for text. Overridden
		here because font style affects the width of the text. We must
		adjust the Caption to the size of the text.
 ******************************************************************************/

void	CCaption::SetFontStyle(
	short		aStyle)
{
	inherited::SetFontStyle(aStyle);
	
	autoRefresh = TRUE;					/* Must redraw all text				*/
	FitToText();
	autoRefresh = FALSE;
}
	

/******************************************************************************
 AdjustBounds {OVERRIDE}

		Adjust the bounds of a Caption to match the associated TextEdit record.
 ******************************************************************************/

void	CCaption::AdjustBounds()
{
	if (firstCall) {
		inherited::AdjustBounds();
	
		firstCall = FALSE;
		FitToText();
		firstCall = TRUE;
	}
}


/*=============================================================================
 TEClickLoop

		Do nothing special when clicking inside text
 =============================================================================*/
		
static pascal Boolean	TEClickLoop()
{
	return(TRUE);
}
