
/*
 *  iomisc.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#include "stdio.h"
#include "stdarg.h"
#include "stdlib.h"
#include "string.h"
#include "errno.h"
#include "ansi_private.h"


#ifdef _NOCONSOLE_
FILE __file[FOPEN_MAX];
#else
FILE __file[FOPEN_MAX] = { { 1 }, { 1 }, { 1 } };
#endif


/*
 *  __getfile - obtain an unused file record
 *
 *  The first three records are reserved for stdin, stdout, and stderr
 *  respectively.  NULL is returned if all other records are in use.
 *
 */

FILE *
__getfile(void)
{
	register FILE *fp;
	int n;
	
	for (fp = &__file[3], n = FOPEN_MAX - 3; n--; fp++) {
		if (!fp->refnum)
			return(fp);
	}
	errno = EMFILE;
	return(NULL);
}


/* ---------- printf/scanf ---------- */


int
fprintf(FILE *fp, const char *fmt, ...)
{
	return(vfprintf(fp, fmt, __va(fmt)));
}


int
printf(const char *fmt, ...)
{
	return(vprintf(fmt, __va(fmt)));
}


int
vprintf(const char *fmt, void *p)
{
	return(vfprintf(stdout, fmt, p));
}


int
fscanf(FILE *fp, const char *fmt, ...)
{
	return(_vfscanf(fp, fmt, __va(fmt)));
}


int
scanf(const char *fmt, ...)
{
	return(_vscanf(fmt, __va(fmt)));
}


int
_vscanf(const char *fmt, void *p)
{
	return(_vfscanf(stdin, fmt, p));
}


void
setbuf(FILE *fp, char *buf)
{
	setvbuf(fp, buf, buf ? _IOFBF : _IONBF, BUFSIZ);
}


int
setvbuf(FILE *fp, char *buf, int mode, size_t size)
{
	int mybuf = 0;
	
	__checkfile(fp);
	switch (mode) {
		case _IONBF:
			buf = &fp->one;
			size = 1;
			break;
		case _IOLBF:
		case _IOFBF:
			if (size) {
				if (buf)
					break;
				mybuf = 1;
				if (buf = malloc(size))
					break;
			}
			/* ... */
		default:
			return(-1);
	}
	if (fp->mybuf) {
		free(fp->buf);
		fp->mybuf = 0;
	}
	if (mybuf)
		fp->mybuf = 1;
	fp->buf = buf;
	fp->size = size;
	__reset(fp);
	return(0);
}


int
fflush(FILE *fp)
{
	int result, n;
	
	if (fp) {
		__checkfile(fp);
		return(__flush(fp));
	}
	result = 0;
	for (fp = &__file[0], n = FOPEN_MAX; n--; fp++) {
		if (fp->dirty && __flush(fp))
			result = EOF;
	}
	return(result);
}


int
fclose(FILE *fp)
{
	int result = 0;

	fp->std = 0;
	if (fp->refnum) {
		if (__flush(fp))
			result = EOF;
		if (__close(fp))
			result = EOF;
		if (fp->mybuf)
			free(fp->buf);
		memset(fp, 0, sizeof(FILE));
	}
	return(result);
}
