
/*
 *  stdio.c
 *
 *  Copyright (c) 1991 Symantec Corporation.  All rights reserved.
 *
 */

#include "stdio.h"
#include "stdarg.h"
#include "string.h"
#include "limits.h"
#include "ansi_private.h"

static void initfp(FILE *, char *, size_t);
static int nullio(FILE *, int);

char __copyright[] = "THINK C Libraries  1991 Symantec Corp.";


/*
 *  __checkfile - open std stream if necessary
 *
 *  If the stream is an unopened std stream, it (as well as the other std
 *  streams) is opened, thus providing the illusion that it was open all
 *  along.
 *
 *  If the stream is not open, we try to make it look enough like an open
 *  stream so that we won't crash.  Reads will return EOF, and writes will
 *  go to the bit-bucket.
 *
 */

FILE *
__checkfile(FILE *fp)
{
#ifndef _NOCONSOLE_
	if (fp->std)
		__open_std();
#endif
	if (!fp->refnum) {
		fp->ptr = (unsigned char *) (fp->buf = &fp->one);
		fp->size = 1;
		fp->proc = nullio;
	}
	return(fp);
}


/*
 *  __c2p - convert string from C to pascal format
 *
 */

unsigned char *
__c2p(register const char *s, register char *t)
{
	asm {
		movea.l	t,a0			;  A0 ==> start of p-string
		st		d1				;  D1.B = length
@1		move.b	d0,(t)+
		addq.b	#1,d1
		move.b	(s)+,d0
		bne.s	@1
		move.b	d1,(a0)
		move.l	a0,d0			;  return p-string
	}
}


/*
 *  __strin - set up an input stream for sscanf
 *
 */

FILE *
__strin(FILE *fp, const char *s)
{
	initfp(fp, (char *) s, strlen(s));
	return(fp);
}


/*
 *  __strout - set up an output stream for sprintf
 *
 */

FILE *
__strout(FILE *fp, char *s)
{
	initfp(fp, s, ULONG_MAX);
	fp->dirty = 1;
	return(fp);
}


/*
 *  initfp - initialize a file pointer to dummy values
 *
 */

static void
initfp(FILE *fp, char *s, size_t cnt)
{
	memset(fp, 0, sizeof(FILE));
	fp->refnum = -1;
	fp->ptr = (unsigned char *) s;
	fp->cnt = cnt;
	fp->proc = nullio;
}


/*
 *  nullio - dummy I/O proc
 *
 */

static int
nullio(FILE *fp, int i)
{
	return(EOF);
}


/* ---------- sprintf/sscanf ---------- */


int
sprintf(char *s, const char *fmt, ...)
{
	return(vsprintf(s, fmt, __va(fmt)));
}


int
vsprintf(char *s, const char *fmt, void *p)
{
	FILE f;
	int n;
	
	if ((n = vfprintf(__strout(&f, s), fmt, p)) >= 0)
		s[n] = 0;
	return(n);
}


int
sscanf(const char *s, const char *fmt, ...)
{
	return(_vsscanf(s, fmt, __va(fmt)));
}


int
_vsscanf(const char *s, const char *fmt, void *p)
{
	FILE f;
	
	return(_vfscanf(__strin(&f, s), fmt, p));
}


/* ---------- vectored interface ---------- */


int
__read(FILE *fp)
{
	return((*fp->proc)(fp, 0));
}


int
__write(FILE *fp)
{
	return((*fp->proc)(fp, 1));
}


int
__close(FILE *fp)
{
	return((*fp->proc)(fp, 2));
}
