
/*
 *  DA main.c
 *
 *  Copyright (c) 1989 Symantec Corporation.  All rights reserved.
 *
 *  This file lets you use the source-level debugger to develop a desk
 *  accessory.  (System version 6.0 is required.)
 *
 *  The source-level debugger only works on applications, so the trick
 *  is to embed your DA in an application whose only job is to run DA's.
 *
 *  To use this technique, write your DA exactly as you normally would,
 *  with these exceptions:
 *
 *		1)	Name your main program "_main" rather than "main".
 *		2)  Obey certain restrictions on the use of "SetUpA4" and
 *			"RestoreA4".  (These are discussed below.)  Or, don't
 *			use them at all.
 *		3)  If you're writing a multi-segment DA, use "UnloadSeg"
 *			rather than "UnloadA4Seg".
 *
 *  Prepare your project as follows:
 *
 *		1)  Set the project type to "Application".  The MultiFinder
 *			attribute flags should all be off.
 *		2)	Add this file ("DA main.c") to the project.
 *		3)	Turn on the "Use Debugger" option.
 *		4)  Copy the resources from the file "test DA" into your
 *			project's associated resource file ("<projname>.rsrc").
 *
 *  That's it!  You can now debug your DA, which appears embedded in an
 *  application with standard "Apple", "File", and "Edit" menus.  Your
 *  DA is in the "Apple" menu as "Test DA".  A quicker way to open your
 *  DA is to choose "Open" from the "File" menu.
 *
 *  When you're ready to make a real DA, all you have to do is:
 *
 *		1)	Change the project type back to "Desk Accessory".
 *		2)	Rename your main program back to "main".
 *		3)	Remove this file ("DA main.c") from the project.
 *		4)	Make sure "SetUpA4" and "RestoreA4" are called in
 *			the proper places.
 *		5)	Change your "UnloadSeg" calls back to "UnloadA4Seg".
 *
 *  (It is not necessary to remove the resources that came from the
 *  file "test DA".)
 *
 *  A NOTE ON RESOURCE NUMBERING:  Every DA has a unit number which is
 *  also the resource ID of its 'DRVR' resource.  The resource ID of
 *  any resource "owned" by a DA is a function of the unit number, as
 *  described in the Resource Manager chapter of Inside Macintosh,
 *  volume I.  It is critical that you number your DA's "owned" resources
 *  based on a unit number of 12, as this is the resource ID of the 'DRVR'
 *  resource contained in the file "test DA".  Similarly, when you change
 *  the project type back to "Desk Accessory", you must also set the
 *  resource ID to be 12.  If you want to use a different unit number,
 *  you must rebuild "test DA".
 *
 *  A NOTE ON "SetUpA4"/"RestoreA4":  When you follow the technique
 *  described here to embed your DA in an application, your DA's globals
 *  are addressed using A5, and A4 is available as a register variable.
 *  Therefore "SetUpA4" and "RestoreA4" are not really necessary.  You
 *  could leave them out, but then you will have to put them back in
 *  when you convert to a real DA.  Using "SetUpA4" and "RestoreA4" in
 *  the application-based DA is harmless, except that it is important
 *  not to call "SetUpA4" or "RestoreA4" at a time when A4 holds the
 *  value of a register variable which should not be trashed.  A sufficient
 *  precaution is to always call "SetUpA4" and "RestoreA4" as the very
 *  first and last (respectively) actions in a function; in particular,
 *  wait to initialize register variables until after "SetUpA4" is called.
 *
 */

#ifndef _MacTypes_
#include <MacHeaders>
#endif

static short refnum;					/*  resource file containing DA  */
static Str255 DAname;				/*  name of DA  */
static MenuHandle appleMenu;		/*  Apple menu  */
static long menuChoice;				/*  DA menu selection  */

#define hiword(x)		(((short *) &(x))[0])
#define loword(x)		(((short *) &(x))[1])


main()
{
	short i;
	Handle r;
	long type;
	extern myMain();
	MenuHandle menu;
	EventRecord event;

		/*  initialize  */
		
	MaxApplZone();
	for (i = 0; i < 10; i++)
		MoreMasters();
	InitGraf(&thePort);
	InitCursor();
	InitFonts();
	InitWindows();
	TEInit();
	InitDialogs(0);
	InitMenus();
	FlushEvents(everyEvent, 0);
		
		/*  look up DA  */
	
	refnum = CurResFile();
	r = Get1IndResource('DRVR', 1);
	if (!r)
		return;
	GetResInfo(r, &i, &type, DAname);
	r = Get1IndResource('DATA', 1);
	if (!r)
		return;
	* (ProcPtr *) *r = (ProcPtr)myMain;
	ChangedResource(r);
	WriteResource(r);

		/*  build apple menu  */

	InsertMenu(appleMenu = NewMenu(1, "\p\024"), 0);
	AddResMenu(appleMenu, 'DRVR');
	resolve_ID_conflict(i);
	
		/*  build remaining menus  */
		
	InsertMenu(menu = NewMenu(2, "\pFile"), 0);
	AppendMenu(menu, "\pOpen/O;Close;Quit/Q");
	InsertMenu(menu = NewMenu(3, "\pEdit"), 0);
	AppendMenu(menu, "\pUndo/Z;(-;Cut/X;Copy/C;Paste/V;Clear");
	DrawMenuBar();

		/*  process events  */
	
	for (;;) {
		HiliteMenu(0);
		SystemTask();
		UseResFile(refnum);
		SEvtEnb = false;
		if (GetNextEvent(everyEvent, &event)) {
			if (!SystemEvent(&event))
				doEvent(&event);
		}
		else if (event.what == nullEvent) {
			if (FrontWindow() == 0)
				InitCursor();
		}
		if (menuChoice) {
			MBarEnable = TheMenu;
			SystemMenu(menuChoice);
			MBarEnable = 0;
			menuChoice = 0;
		}
	}
}


/*
 *  doEvent - handle event
 *
 */

static
doEvent(event)
register EventRecord *event;
{
	short part;
	WindowPeek wp;
	long aLong = 0x00640064;
	switch (event->what) {
		case diskEvt:
			if (hiword(event->message)) {
				InitCursor();
				DIBadMount(*(Point *) aLong, event->message);
			}
			break;
		case mouseDown:
			switch (part = FindWindow(event->where, &wp)) {
				case inMenuBar:
					InitCursor();
					doMenu(MenuSelect(event->where));
					break;
				case inSysWindow:
					SystemClick(event, wp);
					break;
			}
			break;
		case keyDown:
			if (event->modifiers & cmdKey)
				doMenu(MenuKey(loword(event->message)));
			break;
	}
}


/*
 *  doMenu - process menu selection
 *
 */

static
doMenu(choice)
long choice;
{
	short i = loword(choice);
	Str255 buf;
	WindowPeek wp;
	
	switch (hiword(choice)) {
		case 1:		/*  Apple  */
			GetItem(appleMenu, i, buf);
			OpenDeskAcc(buf);
			break;
		case 2:		/*  File  */
			switch (i) {
				case 1:		/*  Open  */
					OpenDeskAcc(DAname);
					break;
				case 2:		/*  Close  */
					if (wp = (WindowPeek) FrontWindow())
						CloseDeskAcc(wp->windowKind);
					break;
				case 3:		/*  Quit  */
					ExitToShell();
			}
			break;
		case 3:		/*  Edit  */
			SystemEdit(i - 1);
			break;
	}
}


/*
 *  resolve_ID_conflict - avoid ID conflict with another DA
 *
 */

static
resolve_ID_conflict(i)
{
	Handle r;
	long type;
	Str255 buf;
	
		/*  don't bother if we're running Suitcase  */
		
	if (((long *) *SysMapHndl)[4])
		return;
		
		/*  look for conflict in System file  */
		
	UseResFile(0);
	SetResLoad(false);
	r = GetResource('DRVR', i);
	SetResLoad(true);
	UseResFile(refnum);
	
		/*  resolve conflict by removing old DA from menu  */

	if (r) {
		GetResInfo(r, &i, &type, buf);
		deleteDA(buf);
	}
}


/*
 *  deleteDA - remove a DA from the Apple menu
 *
 */

static
deleteDA(target)
Str255 target;
{
	short i, n = CountMItems(appleMenu);
	Str255 buf;
	
	for (i = 1; i <= n; i++) {
		GetItem(appleMenu, i, buf);
		if (EqualString(buf, target, false, true)) {
			DelMenuItem(appleMenu, i);
			return;
		}
	}
}


/*
 *  myMain - interface to _main
 *
 *  All calls directed to the DA pass through here.  Menu selections
 *  are deferred.
 *
 */

static
myMain()
{
	extern _main();
	
	asm {
		pea		_main
		cmpi.w	#2,16(sp)
		bne.s	@1
		movea.l	8(sp),a0
		cmpi.w	#accMenu,OFFSET(cntrlParam,csCode)(a0)
		bne.s	@1
		tst.l	menuChoice
		bne.s	@1
		move.l	OFFSET(cntrlParam,csParam)(a0),menuChoice
		addq.l	#4,sp
@1	}
}
