
/*
 *  DA shell.c
 *
 *  Copyright (c) 1989 Symantec Corporation.  All rights reserved.
 *
 *  This is the source to "DA shell", the program that is launched
 *  by the "Run" command in a desk accessory project.
 *
 *  When used with MultiFinder, version 6.0 of the System is required.
 *
 *  "DA shell" is an application with standard "Apple", "File", and
 *  "Edit" menus.  Your DA appears in the "Apple" menu.  A quicker way
 *  to open your DA is to choose "Open" from the "File" menu.
 *
 *  "DA shell" does NOT allow you to use the source-level debugger to
 *  develop your DA.  To do that, you must use the technique described
 *  in the comments in the file "DA main.c".
 *
 */

static short refnum;					/*  resource file containing DA  */
static Str255 DAname;				/*  name of DA  */
static MenuHandle appleMenu;		/*  Apple menu  */

#define hiword(x)		(((short *) &(x))[0])
#define loword(x)		(((short *) &(x))[1])


main()
{
	short i, code, count;
	AppFile appFile;
	Handle r;
	long type;
	MenuHandle menu;
	EventRecord event;

		/*  initialize  */
		
	MaxApplZone();
	for (i = 0; i < 10; i++)
		MoreMasters();
	InitGraf(&thePort);
	InitCursor();
	InitFonts();
	InitWindows();
	TEInit();
	InitDialogs(0);
	InitMenus();
	FlushEvents(everyEvent, 0);
	
		/*  open DA file  */
		
	CountAppFiles(&code, &count);
	if (code != appOpen || count != 3)
		return;
	GetAppFiles(1, &appFile);
	if (appFile.fType != 'DFIL')
		return;
	refnum = OpenRFPerm(appFile.fName, appFile.vRefNum, fsRdWrPerm);
	if (refnum < 0)
		return;
		
		/*  look up DA  */
	
	r = Get1IndResource('DRVR', 1);
	if (!r)
		return;
	GetResInfo(r, &i, &type, DAname);

		/*  build apple menu  */

	InsertMenu(appleMenu = NewMenu(1, "\p\024"), 0);
	AddResMenu(appleMenu, 'DRVR');
	resolve_ID_conflict(i);
	
		/*  build remaining menus  */
		
	InsertMenu(menu = NewMenu(2, "\pFile"), 0);
	AppendMenu(menu, "\pOpen/O;Close;Quit/Q");
	InsertMenu(menu = NewMenu(3, "\pEdit"), 0);
	AppendMenu(menu, "\pUndo/Z;(-;Cut/X;Copy/C;Paste/V;Clear");
	DrawMenuBar();

		/*  process events  */
	
	for (;;) {
		HiliteMenu(0);
		SystemTask();
		UseResFile(refnum);
		if (GetNextEvent(everyEvent, &event))
			doEvent(&event);
		else if (event.what == nullEvent) {
			if (FrontWindow() == 0)
				InitCursor();
		}
	}
}


/*
 *  doEvent - handle event
 *
 */

static
doEvent(event)
register EventRecord *event;
{
	short part;
	WindowPeek wp;
	long aLong = 0x00640064;
	
	switch (event->what) {
		case diskEvt:
			if (hiword(event->message)) {
				InitCursor();
				DIBadMount(*(Point*)&aLong, event->message);
			}
			break;
		case mouseDown:
			switch (part = FindWindow(event->where, &wp)) {
				case inMenuBar:
					InitCursor();
					doMenu(MenuSelect(event->where));
					break;
				case inSysWindow:
					SystemClick(event, wp);
					break;
			}
			break;
		case keyDown:
			if (event->modifiers & cmdKey)
				doMenu(MenuKey(loword(event->message)));
			break;
	}
}


/*
 *  doMenu - process menu selection
 *
 */

static
doMenu(choice)
long choice;
{
	short i = loword(choice);
	WindowPeek wp;
	AppFile appFile;
	
	switch (hiword(choice)) {
		case 1:		/*  Apple  */
			GetItem(appleMenu, i, appFile.fName);
			OpenDeskAcc(appFile.fName);
			break;
		case 2:		/*  File  */
			switch (i) {
				case 1:		/*  Open  */
					OpenDeskAcc(DAname);
					break;
				case 2:		/*  Close  */
					if (wp = (WindowPeek) FrontWindow())
						CloseDeskAcc(wp->windowKind);
					break;
				case 3:		/*  Quit  */
					GetAppFiles(2, &appFile);
					if (appFile.fType == 'APPL') {
						SetVol(0, appFile.vRefNum);
						Launch(0, appFile.fName);
					}
					ExitToShell();
			}
			break;
		case 3:		/*  Edit  */
			SystemEdit(i - 1);
			break;
	}
}


/*
 *  resolve_ID_conflict - avoid ID conflict with another DA
 *
 */

static
resolve_ID_conflict(i)
{
	Handle r;
	long type;
	Str255 buf;
	
		/*  don't bother if we're running Suitcase  */
		
	if (((long *) *SysMapHndl)[4])
		return;
		
		/*  look for conflict in System file  */
		
	UseResFile(0);
	SetResLoad(false);
	r = GetResource('DRVR', i);
	SetResLoad(true);
	UseResFile(refnum);
	
		/*  resolve conflict by removing old DA from menu  */

	if (r) {
		GetResInfo(r, &i, &type, buf);
		deleteDA(buf);
	}
}


/*
 *  deleteDA - remove a DA from the Apple menu
 *
 */

static
deleteDA(target)
Str255 target;
{
	short i, n = CountMItems(appleMenu);
	Str255 buf;
	
	for (i = 1; i <= n; i++) {
		GetItem(appleMenu, i, buf);
		if (EqualString(buf, target, false, true)) {
			DelMenuItem(appleMenu, i);
			return;
		}
	}
}
