/******************************************************************************
 CBorder.c

							The Border Class
		
	SUPERCLASS = CPane
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CBorder.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IBorder

		Initialize a Border object
 ******************************************************************************/

void	CBorder::IBorder(
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor,
	short			aWidth,
	short			aHeight,
	short			aHEncl,
	short			aVEncl,
	SizingOption	aHSizing,
	SizingOption	aVSizing)
{
	CPane::IPane(anEnclosure, aSupervisor,
					 aWidth, aHeight, aHEncl, aVEncl, aHSizing, aVSizing);
					 
	thickness = 1;
	dropShadow = 0;
}
	

/******************************************************************************
 IViewTemp {OVERRIDE}

		Initialize a Border object using a template
 ******************************************************************************/

void	CBorder::IViewTemp(
	CView			*anEnclosure,		/* Enclosing view					*/
	CBureaucrat		*aSupervisor,		/* Boss in chain of command			*/
	Ptr				viewData)			/* Template with View data			*/
{
	register BorderTempP	p;
	
	p = (BorderTempP) viewData;			/* Initialize superclass			*/
	inherited::IViewTemp(anEnclosure, aSupervisor, (Ptr) &p->sPaneTemp);
	
	thickness = p->thickness;			/* Set instance vars from template	*/
	dropShadow = p->dropShadow;
}


/**** A C C E S S I N G   M E T H O D S ****/


/******************************************************************************
 SetThickness

		Specify the thickness of a Border
 ******************************************************************************/

void	CBorder::SetThickness(
	short		aThickness)
{
	thickness = aThickness;
}


/******************************************************************************
 SetDropShadow

		Specify the length of a drop shadow for a Border
 ******************************************************************************/

void	CBorder::SetDropShadow(
	short		aDropShadow)
{
	dropShadow = aDropShadow;
}


/******************************************************************************
 SetInteriorSize

		Specify the size of the interior area of a Border. Frame size is
		adjusted so that the interior has the specified dimensions.
		The Border is NOT automatically redrawn.
 ******************************************************************************/

void	CBorder::SetInteriorSize(
	short		wInterior,				/* Width for interior in pixels		*/
	short		hInterior)				/* Height for interior in pixels	*/
{
	LongRect	interior;
	Rect		delta;
	
	GetInterior(&interior);
	delta.right = wInterior - (interior.right - interior.left);
	delta.bottom = hInterior - (interior.bottom - interior.top);
	delta.left = delta.top = 0;
	
	ChangeSize(&delta, FALSE);
}


/******************************************************************************
 GetInterior {OVERRIDE}

		Retrieve the interior of a Border. Interior is smaller than the
		frame due to thickness and drop shadow of the Border.
 ******************************************************************************/

void	CBorder::GetInterior(
	register LongRect		*theInterior)
{
	theInterior->left = frame.left + thickness;
	theInterior->top = frame.top + thickness;
	theInterior->right = frame.right - thickness - dropShadow;
	theInterior->bottom = frame.bottom - thickness - dropShadow;
}


/**** D R A W I N G   M E T H O D S ****/


/******************************************************************************
 Draw

		Draw a Border
 ******************************************************************************/

void	CBorder::Draw(
	Rect		*area)
{
	Rect		r;
	PenState	savedState;
	
	GetPenState( &savedState);
	FrameToQDR( &frame, &r);
		
	r.right -= dropShadow;
	r.bottom -= dropShadow;
	PenNormal();
	PenSize(thickness, thickness);
	FrameRect(&r);
	
	if (dropShadow > 0) {
		PenSize(dropShadow, dropShadow);
		MoveTo(frame.left + dropShadow, frame.bottom - dropShadow);
		LineTo(frame.right - dropShadow, frame.bottom - dropShadow);
		LineTo(frame.right - dropShadow, frame.top + dropShadow);
	}
	
	SetPenState( &savedState);
}