/******************************************************************************
 CRadioButton.c

							The RadioButton Class
		
	A standard Mac radio button control. A radio button has a small circle
	which may be open or filled and some text to the right of the circle.
	Radio buttons usually come in groups (see CRadioGroup) where only one
	button in a group can be selected at a time.
	
	SUPERCLASS = CButton
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CRadioButton.h"
#include "Commands.h"
#include "CRadioGroup.h"


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IRadioButton

		Initialize a RadioButton object
 ******************************************************************************/

void	CRadioButton::IRadioButton(
	short			CNTLid,
	CView			*anEnclosure,
    CRadioGroup     *aSupervisor)
{
	CButton::IButton(CNTLid, anEnclosure, aSupervisor);
	idNumber = CNTLid;
}
	

/******************************************************************************
 DoGoodClick {OVERRIDE}

		A RadioButton responds to a good click (the mouse being pressed
		and released within the button) by relaying a command to its
		supervisor if it is not already selected. 
 ******************************************************************************/

void	CRadioButton::DoGoodClick(
	short		whichPart)				/* Will always be inButton			*/
{
	if (GetCtlValue(macControl) == BUTTON_OFF)
		((CRadioGroup*)itsSupervisor)->ChangeStation(this);

	if (clickCmd != cmdNull)			/* Issue the appropriate command	*/
		itsSupervisor->DoCommand(clickCmd);
}
	

/******************************************************************************
 GetIdNumber

		Return id number of a RadioButton
 ******************************************************************************/

short	CRadioButton::GetIdNumber()
{
	return(idNumber);					/* Return instance variable			*/
}
