/******************************************************************************
 CArrowPopupPane.c

		Subclass of CPopupPane that implements a popup menu with
		a simple down-arrow and shadow box.
		
	SUPERCLASS = CPopupPane
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CArrowPopupPane.h"
#include "CPaneBorder.h"
#include "Constants.h"
#include "TBUtilities.h"

#define	kPaneSize		20				/* height and width of pane */

/******************************************************************************
 IArrowPopupPane
 
 	Initialize the ArrowPopupPane. MenuID specifies the resource ID of
 	the menu, fAutoSelect controls whether items are automatically
 	checked, and fMultiSelect controls whether multiple selections
 	are allowed.
******************************************************************************/

void CArrowPopupPane::IArrowPopupPane( short menuID,
			Boolean fAutoSelect, Boolean fMultiSelect,
			CView *anEnclosure, CBureaucrat *aSupervisor,
			short aHEncl, short aVEncl)
{
	CPopupPane::IPopupPane( menuID, fAutoSelect, fMultiSelect, anEnclosure,
				aSupervisor, kPaneSize, kPaneSize, aHEncl, aVEncl,
				sizFIXEDSTICKY, sizFIXEDSTICKY);
				
	IArrowPopupPaneX();	

}	/* CArrowPopupPane::IArrowPopupPane */

/******************************************************************************
 IViewTemp {OVERRIDE}
 
 	Initialize the ArrowPopup from a resource.
******************************************************************************/

void CArrowPopupPane::IViewTemp( CView *anEnclosure, CBureaucrat *aSupervisor,
						Ptr	viewData)
{
	inherited::IViewTemp( anEnclosure, aSupervisor, viewData);
	IArrowPopupPaneX();

}	/* CArrowPopupPane::IViewTemp */

/******************************************************************************
 IArrowPopupPaneX
 
 	Initialization common to IArrowPopupPane and IViewTemp.
******************************************************************************/

void CArrowPopupPane::IArrowPopupPaneX( void)
{
	CPaneBorder		*border;
			
	border = new( CPaneBorder);
	border->IPaneBorder( kBorderFrame);
	border->SetShadow( 2, 2, 1, 1);

	SetBorder( border);

}	/* CArrowPopupPane::IArrowPopupPaneX */

/******************************************************************************
 Draw {OVERRIDE}
 
 	Draw the popup box.
******************************************************************************/

void CArrowPopupPane::Draw( Rect *area)
{
	Point	sicnPt;
	
	sicnPt.h = 4;
	sicnPt.v = 7;
	DrawSICN( TCL_SICN, POPUP_SICN, sicnPt);
	
	inherited::Draw( area);

}	/* CArrowPopupPane::Draw */

/******************************************************************************
 CalcPopupPoint {OVERRIDE}
 
 	Calculate the popup position in global coordinates.
 
******************************************************************************/

Point CArrowPopupPane::CalcPopupPoint( Point hitPt, short modifierKeys, long when)
{
	Point pt;
	
	pt.h = pt.v = 0;
	LocalToGlobal( &pt);
	return pt;

}	/* CArrowPopupPane::CalcPopupPoint */
