/******************************************************************************
 CRadioGroupPane.c

		A CRadioGroupPane is a pane that manages CRadioControl objects.
		When a RadioGroupPane is the supervisor of a group of
		CRadioControls, it will automatically ensure that only
		one member of the group is selected at a time.
		The CRadioGroupPane is also useful for for providing a visual
		grouping of the radio controls by placing them all within
		the CRadioGroupPane and placing a border around it.
		
	SUPERCLASS = CPane
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CRadioGroupPane.h"
#include "CRadioControl.h"

/******************************************************************************
 IRadioGroupPane
 
 	Initialize a RadioGroupPane
******************************************************************************/

void CRadioGroupPane::IRadioGroupPane( CView *anEnclosure, CBureaucrat *aSupervisor,
				short aWidth, short aHeight, short aHEncl, short aVEncl,
				SizingOption aHSizing, SizingOption aVSizing)
{
	CPane::IPane( anEnclosure, aSupervisor, aWidth, aHeight, aHEncl, aVEncl,
				aHSizing, aVSizing);
	
	currentStation = NULL;

}	/* CRadioGroupPane::IRadioGroupPane */


/******************************************************************************
 IViewTemp
 
 	Initialize a RadioGroupPane from a resource template
******************************************************************************/

void CRadioGroupPane::IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
						Ptr viewData)
{
	inherited::IViewTemp( anEnclosure, aSupervisor, viewData);
	currentStation = NULL;
	
}	/* CRadioGroupPane::IViewTemp */

/******************************************************************************
 ProviderChanged
 
 	Respond to a change in a provider. The only provider this class
 	knows about is CRadioControl. When it receives notification that
 	a radio control has been selected, the RadioGroupPane turns off
 	the previously selected control and updates the currentStation
 	instance variable to refer to the newly selected button.
******************************************************************************/

void CRadioGroupPane::ProviderChanged( CCollaborator *aProvider,
				long reason, void* info)
{
	if (member( aProvider, CRadioControl))
	{
		if ((reason == controlValueChanged) && ((*(short*)info) == BUTTON_ON))
		{
			if (currentStation)
				currentStation->SetValue( BUTTON_OFF);
			currentStation = (CRadioControl*) aProvider;
		}
	}
	inherited::ProviderChanged( aProvider, reason, info);

}	/* CRadioGroupPane::ProviderChanged */

/******************************************************************************
 SetStationID
 
 	Set the current station to the RadioControl with the given view ID
******************************************************************************/

void CRadioGroupPane::SetStationID( long aStationID)
{
	CRadioControl *radio;
	
	if (currentStation)
	{
		if (currentStation->GetID() == aStationID)
			return;
			
		currentStation->SetValue( BUTTON_OFF);
	}
	radio = (CRadioControl*) FindViewByID( aStationID);
	if (radio)
	{
		ASSERT( member( radio, CRadioControl));
		radio->SetValue( BUTTON_ON);
	}
	currentStation = radio;

}	/* CRadioGroupPane::SetStationID */

/******************************************************************************
 GetStationID
 
 	Get the view ID of the current RadioControl
******************************************************************************/

long CRadioGroupPane::GetStationID( void)
{
	return currentStation? currentStation->GetID() : 0;

}	/* CRadioGroupPane::GetStationID */