/******************************************************************************
 CClipboard.h

		Interface for the Clipboard Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- added EmptyGlobalScrap, EmptyScrap methods. PutGlobalScrap
		  no longer calls ZeroScrap, so you must call EmptyGlobalScrap
		  or EmptyScrap before calling PutGlobalScrap or PutData.
		- added MakeClipView method
	]

 ******************************************************************************/
 
#define _H_CClipboard

#include "CDirector.h"					/* Interface for its superclass		*/

typedef enum {
	GLOBAL_SCRAP_NEWER,
	PRIVATE_SCRAP_NEWER,
	SCRAPS_THE_SAME
} ScrapStatus;

class CApplication;
class CPanorama;
class CScrollPane;

class CClipboard : public CDirector {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	CPanorama	*itsContents;			/* Pane for displaying contents		*/
	CScrollPane	*itsScrollPane;			/* Contents can be scrolled			*/
	long		theLength;				/* Length from last Get operation	*/
	long		theOffset;				/* Offset from last Get operation	*/
	short		lastScrapCount;			/* Count at the last conversion		*/
										/*   between global and private		*/
	Boolean		privateNewer;			/* Has private scrap changed since	*/
										/*   last conversion?				*/
	Boolean		windowVisible;			/* Is clipboard window visible?		*/
										
								/** Instance Methods **/
                                
    
    void        IClipboard(  CApplication *aSupervisor, Boolean hasWindow);

	virtual void		Suspend(void);
	virtual void		Resume(void);
	virtual Boolean		Close(Boolean quitting);
	virtual void		CloseWind(CWindow *theWindow);
	
	virtual void		Toggle(void);
	virtual void		PutGlobalScrap(ResType theType, Handle theData);
	virtual Boolean		GetGlobalScrap(ResType theType, Handle theData);
	virtual ScrapStatus	Status(void);
	virtual void		ScrapConverted(void);
	virtual void		ConvertGlobal(void);
	virtual void		ConvertPrivate(void);
	virtual void		EmptyGlobalScrap(void);
	virtual void		EmptyScrap(void);
	virtual void		PutData(ResType theType, Handle theData);
	virtual Boolean		GetData(ResType theType, Handle *theData);
	virtual long		DataSize(ResType theType);
	virtual void		PrivateChanged(void);
	virtual void		UpdateDisplay(void);
	virtual CPanorama*	MakeClipView( long dataType, Handle dataHandle);
};