/******************************************************************************
 CCluster.h

		Interface for the Cluster Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- CCluster is now a subclass of CArray. CArray already has
		  the instance variables blockSize and slots,  so
		  these are removed from CCluster's declaration. CArray also
		  has an items handle, but it is declared as a handle instead
		  of LongHandle. For backward compatibility, the items instance
		  variable is maintained, but as an alias to CArray's handle.
		  This also lets certain operations be implemented more efficiently.
		- The Dispose, MoreSlots, and SetBlockSize methods have been
		  removed because they are now implemented by CArray.
		- moved BAD_INDEX constant to CArray.h
		- added prototypes for the function pointer arguments for
		  DoForEach, DoForEach1, FindItem, and FindItem1 methods.
		  Changed declarations of those methods to use new prototypes.
		- added new typedefs EachFunc, EachFunc1, TestFunc, and TestFunc1
		  for function pointer arguments to DoForEach, DoForEach1, FindItem,
		  and FindItem1 methods. Changed prototypes for those functions
		  to accept new types. 
		- added override of Copy
	]

 ******************************************************************************/
 
#define _H_CCluster

#include "CArray.h"				/* Interface for its superclass		*/
#include "Global.h"

	/* prototypes for DoForEach, DoForEach1, FindItem, and FindItem1 function ptrs	*/

typedef void (*EachFunc)(...);
typedef void (*EachFunc1)(...);
typedef Boolean (*TestFunc)(...);
typedef Boolean (*TestFunc1)(...);

class CCluster : public CArray {			/* Class Declaration				*/

public:
								/** Instance Variables **/
	
	LongHandle	items;			/* same as CArray::hItems					*/
	
								/** Instance Methods **/
	void		ICluster(void);			/* Initialization					*/
	virtual void		DisposeAll(void);		/* Dispose of Cluster and all items	*/
	virtual void		DisposeItems(void);		/* Dispose of all items in Cluster	*/

										/* Add an object to the cluster		*/
	virtual void		Add(CObject *theObject);
										/* Remove an object					*/
	virtual void		Remove(CObject *theObject);
										/* Test for inclusion of an object	*/
	virtual Boolean		Includes(CObject *theObject);
	virtual void		DoForEach( EachFunc);	/* Call a function for each item in	*/
										/*   the cluster					*/
	virtual void		DoForEach1( EachFunc1, long);
	virtual CObject*	FindItem( TestFunc);
	virtual CObject*	FindItem1( TestFunc1, long param);
										/* Find offset of object in cluster	*/
	virtual long		Offset(CObject *theObject);
	
	virtual CObject*	Copy( void);
};

#define		SLOT_SIZE		4			/* Byte size of a slot. A slot		*/
										/*   holds a handle to an object.	*/
