/******************************************************************************
 CPatternGrid.c

							The PatternGrid Class
		
	SUPERCLASS = CGridSelector
	
	Copyright  1989 Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "Global.h"
#include "CPatternGrid.h"
#include "TCLUtilities.h"

typedef struct PtGdRec {				/* Resource template for a			*/
	short		rows;					/*   pattern grid					*/
	short		cols;
	short		boxWidth;
	short		boxHeight;
	short		hSizing;
	short		vSizing;
	short		hLoc;
	short		vLoc;
	short		commandBase;
	short		patListID;
	short		numPats;
	short		thePatterns[];
} PtGdRec, *PtGdPtr, **PtGdHand;


/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/


/******************************************************************************
 IPatternGrid

		Initialize a PatternGrid object
 ******************************************************************************/

void	CPatternGrid::IPatternGrid(
	short			PtGdid,
	CView			*anEnclosure,
	CBureaucrat		*aSupervisor)
{
	PtGdHand			r;
	register PtGdPtr	p;
	
	r = (PtGdHand) GetResource('PtGd', PtGdid);
	FailNILRes( r);
	HLock( (Handle) r);
	p = *r;
	
	CGridSelector::IGridSelector(anEnclosure, aSupervisor,
						p->hLoc, p->vLoc, p->hSizing, p->vSizing,
						1, p->commandBase, p->rows, p->cols,
						p->boxWidth, p->boxHeight);
	
	patListID = p->patListID;
	numPats = p->numPats;
	thePatterns = (short**) NewHandleCanFail((long) numPats * sizeof(short));
	FailNIL( thePatterns);
	
	BlockMove(p->thePatterns, *thePatterns, (Size) numPats * sizeof(short)); 
	
	HUnlock( (Handle) r);
}
	

/******************************************************************************
 Dispose {OVERRIDE}

		Dispose of a pattern grid
 ******************************************************************************/

void	CPatternGrid::Dispose()
{
	DisposHandle( (Handle) thePatterns);/* Get rid of list of pattern ID's	*/
	inherited::Dispose();				/* Pass message on to superclass	*/
}


/******************************************************************************
 DrawItem

		Draw the pattern at the specified row and column
 ******************************************************************************/

void	CPatternGrid::DrawItem(
	short		theItem,
	Rect		*theBox)
{
	Pattern		thePat;
	
	GetIndPattern( thePat, patListID, (*thePatterns)[theItem - 1]);
	FillRect(theBox, thePat);
}
	

/******************************************************************************
 HiliteItem {OVERRIDE}

		Hilite an item in the specified way
 ******************************************************************************/

void	CPatternGrid::HiliteItem(
	short		theItem,				/* Item number to hilite			*/
	HiliteState	state)					/* State of hiliting desired		*/
{
	Rect			theRect;
	register Rect	*theBox;
	long			ticks;
	Rect			hBox;
	
	if (theItem == NOTHING)
		return;

	theBox = &theRect;
	FindItemBox(theItem, &theRect);
	
	switch (state) {
	
		case hiliteON:
			hBox = theRect;
			InsetRect(&hBox, -4, -4);
			PenMode(patCopy);
			PenPat(white);
			PenSize(3,3);
			FrameRect(&hBox);
			InsetRect(&hBox, -1, -1);
			PenNormal();
			FrameRect(&hBox);
			break;
			
		case hiliteOFF:
			hBox = theRect;
			InsetRect(&hBox, -5, -5);
			RefreshRect(&hBox);
			break;
			
		case hiliteDYNAMIC:
			PenNormal();
			PenMode(patXor);
			FrameRect(theBox);
			Delay(2, &ticks);
			InsetRect(theBox, 1, 1);
			FrameRect(theBox);
			Delay(2, &ticks);
			FrameRect(theBox);
			Delay(2, &ticks);
			InsetRect(theBox, -1, -1);
			FrameRect(theBox);
			PenNormal();
			break;
	}
}


/******************************************************************************
 GetPattern

		Return the currently selected pattern
 ******************************************************************************/

void	CPatternGrid::GetPattern(
	Pattern		*thePattern)
{
	GetIndPattern(*thePattern, patListID, (*thePatterns)[selection - 1]);
}