/******************************************************************************
 CAbstractText.h

		Interface for Abstract text editing class
		
	SUPERCLASS = CPanorama
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#define _H_CAbstractText

#include "CPanorama.h"

#define kAlignNotContinuous		MININT	/* alignment not continuous over selection */
#define kAlignJustify			5		/* full justification					   */

enum	// mnemonic constants for the Specify method
{
	kNotEditable = FALSE,
	kEditable,
	
	kNotSelectable = FALSE,
	kSelectable,
	
	kNotStylable = FALSE,
	kStylable	
};

	/* buffer for a single character, allowing for multi-byte scripts	*/
	/* first byte is length-byte, as in a Pascal string					*/
	
typedef unsigned char  tCharBuf[5];

class CTextEditTask;
class CTextStyleTask;

class CAbstractText : public CPanorama
{
public:

	static	short cFirstTaskIndex;		// index in Undo STR# resource of first undo label
										// see further notes below
	CTextEditTask *itsTypingTask; 		// active typing task	
	
	void		IAbstractText(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing,
							short aLineWidth);
							
	virtual void		IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData);
							
									/** Text Specification **/
	virtual void		SetTextString(Str255 textStr);
	virtual void		SetTextHandle(Handle textHand);
	virtual void		SetTextPtr(Ptr textPtr, long numChars);
	virtual Handle		GetTextHandle(void);
	virtual Handle		CopyTextRange( long start, long end);
	virtual void		InsertTextPtr( Ptr text, long length, Boolean fRedraw);
	virtual void		InsertTextHandle( Handle text, Boolean fRedraw);

	virtual void		Specify( Boolean fEditable, Boolean fSelectable, Boolean fStylable);
	virtual void 		GetSpecification( Boolean *fEditable, Boolean *fSelectable, Boolean *fStylable);
	virtual void		SetFontNumber(short aFontNumber);
	virtual void		SetFontName(Str255 aFontName);
	virtual void		SetFontStyle(short aStyle);
	virtual void		SetFontSize(short aSize);
	virtual void		SetTextMode(short aMode);
	virtual void		SetAlignCmd(long anAlignment);
	virtual long		GetAlignCmd( void);
	virtual void		SetSpacingCmd(long aSpacingCmd);
	virtual long		GetSpacingCmd( void);
	
	virtual long		GetHeight( long startLine, long endLine);
	virtual short		Get1Height( long aLineNum);
	virtual long		GetCharOffset( LongPt *aPt);
	virtual void		GetCharPoint( long offset, LongPt *aPt);
	virtual void		GetCharStyle( long charOffset, TextStyle *theStyle);
	virtual void		GetTextStyle( short *whichAttributes, TextStyle *aStyle);
	
	virtual void		GetCharBefore( long *aPosition, tCharBuf charBuf);
	virtual void		GetCharAfter( long *aPosition, tCharBuf charBuf);
	
	
									/** Calibrating **/
	virtual void		ResizeFrame( Rect *deltaRect);
	virtual void		SetWholeLines(Boolean aWholeLines);
	virtual Boolean		GetWholeLines( void);
	
	virtual long		FindLine(long charPos);
	virtual long		GetLength( void);
	virtual long		GetNumLines( void);
	virtual void		DoCommand(long theCommand);
	virtual void		PerformEditCommand( long theCommand);
	virtual void		UpdateMenus(void);

									/** Mouse and Keystrokes **/
	virtual void		DoKeyDown(char theChar, Byte keyCode, EventRecord *macEvent);
	virtual void		DoAutoKey(char theChar, Byte keyCode, EventRecord *macEvent);
	virtual void		TypeChar(char theChar, short theModifiers);
	
	virtual void		SelectionChanged( void);
	virtual void		ScrollToSelection(void);
	
	virtual Boolean		BecomeGopher( Boolean fBecoming);
	virtual void		SetSelection( long selStart, long selEnd, Boolean fRedraw);
	virtual void		GetSelection( long *selStart, long *selEnd);
	virtual void		SelectAll( Boolean fRedraw);
	
	virtual void		Paginate( CPrinter *aPrinter, short pageWidth, short pageHeight);
	
									/** Cursor **/
	virtual void		AdjustCursor(Point where, RgnHandle mouseRgn);
	
protected:

	short		lineWidth;				// Width of a text line in pixels
										//   if <= 0, width is the same as
										//   that of the frame
	short		lastFontNum;			// last font number
	long		lastFontCmd;			// last font cmd #
	short		lastTextSize;			// last seen text size
	long		lastSizeCmd;			// last size cmd #
	Boolean		fixedLineHeights:1;		// TRUE if all lines same height
	Boolean		wholeLines:1;			// Draw only whole lines? i.e.,
										//   don't cut off lines vertically
	Boolean		editable:1;				// TRUE if user can edit text,
										// FALSE text is read-only
	Boolean		stylable:1;				// TRUE if user can change font/size/style
	
									
	
	virtual CTextEditTask *MakeEditTask( long editCmd);
	virtual CTextStyleTask *MakeStyleTask( long styleCmd);

};

// Change protocol for CAbstractText

enum
{
	textSelectionChanged = bureaucratLastChange + 1,	// info parameter is NULL
	
	textLastChange = textSelectionChanged
	
};

/* Usage of cFirstTaskIndex:

	CDocument updates the Undo menu item by sending a GetNameIndex message to the
	most recent task. The integer the task returns is used as an index into the
	STR# 130 resource, and the resulting string is used in the menu item text.
	
	CAbstractText, along with CTextEditTask and CStyleTask can perform a number of
	undoable operations, so more than one string is needed. The tasks assumes
	all the strings are located together in the STR# 130 resource, starting
	at the index given by the class variable cFirstTaskIndex. By default this
	index is 1, meaning these strings are the first in the STR# resource. 
	It also expects them to be in the following order:
		Typing, Cut, Copy, Paste, Clear, Formatting
		
	If you have these strings at a different position in the resource, be sure
	to set cFirstTaskIndex at some point in your application resource. If you
	don't supply the strings at all, set cFirstTaskIndex to 0.

*/

enum	// constants specifying the expected order of undo task names
{
	undoTyping,
	undoCut,
	undoCopy,
	undoPaste,
	undoClear,
	undoFormatting
	
};

typedef struct	{						/* AbstractText template					*/
	PanoramaTemp	sPanoramaTemp;
	short			lineWidth;
	short			wholeLines;
	short			editable;
	short			styleable;
	
} tAbstractTextTemp, *tAbstractTextTempP;
