/******************************************************************************
 CEditText.h

		Interface for the EditText Class
		
		Copyright  1989 Symantec Corporation. All rights reserved.
		
	TCL 1.1 CHANGES
	[
		- BIG CHANGE: EditText is now a subclass of CAbstractText, and
			CStaticText is a subclass of CEditText. Many, many things
			changed because of this. See comments in CEditText.c
	]
	
 ******************************************************************************/
 
#define _H_CEditText

#include "CAbstractText.h"				/* Interface for its superclass		*/

class CEditText : public CAbstractText {		/* Class Declaration				*/

public:
								/** Instance Variables **/

	TEHandle	macTE;			/* Toolbox Text Edit record handle	*/
	long		spacingCmd;		/* Line spacing command number		*/
	long		alignCmd;		/* alignment cmd number				*/

								/** Instance Methods **/
									/** Contruction/Destruction **/
	void		IEditText(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing,
							short aLineWidth);
	virtual void		IViewTemp(CView *anEnclosure, CBureaucrat *aSupervisor,
							Ptr viewData);
	void		IEditTextX(void);
	virtual void		MakeMacTE( void);
	virtual void		Dispose(void);

									/** Mouse and Keystrokes **/
	virtual void		DoClick(Point hitPt, short modifierKeys, long when);
	virtual void		PerformEditCommand( long theCommand);

									/** Display **/
	virtual void		Draw(Rect *area);
	virtual void		Scroll( long hDelta, long vDelta, Boolean redraw);
	virtual void		Activate(void);
	virtual void		Deactivate(void);
	virtual void		SetSelection( long selStart, long selEnd, Boolean fRedraw);
	
									/** Text Specification **/
	virtual void		SetTextPtr(Ptr textPtr, long numChars);
	virtual Handle		GetTextHandle(void);
	virtual Handle 		CopyTextRange( long start, long end);
	virtual void		InsertTextPtr( Ptr text, long length, Boolean fRedraw);

	virtual void		TypeChar(char theChar, short theModifers);
	
	virtual void		CheckInsertion( long insertLen, Boolean useSelection);
	
									/** Calibrating **/
	virtual void		CalcTERects(void);
	virtual void		ResizeFrame(Rect *delta);
	virtual void		AdjustBounds(void);
	
	virtual long		FindLine( long charPos);
	virtual long		GetLength( void);

									/** Text Characteristics **/
	virtual void		SetFontNumber(short aFontNumber);
	virtual void		SetFontStyle(short aStyle);
	virtual void		SetFontSize(short aSize);
	virtual void		SetTextMode(short aMode);
	virtual void		SetAlignment(short anAlignment);
	virtual void		SetAlignCmd( long alignCmd);
	virtual void		SetSpacingCmd( long aSpacingCmd);
	virtual void		GetTEFontInfo(FontInfo *macFontInfo);
	
	virtual long		GetHeight( long startLine, long endLine);
	virtual long		GetCharOffset( LongPt *aPt);
	virtual void		GetCharPoint( long offset, LongPt *aPt);
	virtual void		GetTextStyle( short *whichAttributes, TextStyle *aStyle);
	virtual void 		GetCharStyle( long charOffset, TextStyle *theStyle);
	virtual long		GetSpacingCmd( void);
	virtual long		GetAlignCmd( void);

	virtual long		GetNumLines( void);
	virtual void		GetSelection( long *selStart, long *selEnd);

									/** Printing **/
	virtual void		AboutToPrint(short *firstPage, short *lastPage);
	virtual void		PrintPage(short pageNum, short pageWidth, short pageHeight, CPrinter *aPRinter);
	virtual void		DonePrinting(void);
	
									/** Cursor **/
	virtual void		Dawdle(long *maxSleep);
};


#define		CARRIAGE_RETURN		13		/* Character code						*/
#define		kMaxTELength		32000L  /* maximum text we allow in TE record   */

pascal Boolean	TEClickLoop(void);		// TextEdit callback used for auto-scrolling
extern CEditText *gEditText;			// the currently active CEditText, may be NULL
