/******************************************************************************
 CStyleTEClipboard.c

		A subclass of CClipboard that displays styled text when both 'TEXT'
		and 'styl' data are in the global scrap.
		
		To use this class in your application, you should override 
		CApplication::MakeClipboard, as in:
		
		void	CMyApp::MakeClipboard( void)
		{
			CStyleTEClipboard	*styleClip = new( CStyleTEClipboard);
			
			styleClip->IStyleTEClipboard( this, TRUE);
			gClipboard = styleClip;
		}

		
		
	SUPERCLASS = CClipboard
	
	Copyright  1991 Symantec Corporation. All rights reserved.
	

 ******************************************************************************/

#include "CStyleTEClipboard.h"
#include "CStyleText.h"
#include "CScrollPane.h"
#include "Global.h"

/******************************************************************************
 IStyleTEClipboard
 
 	Initialize a CStyleTEClipboard
******************************************************************************/

void CStyleTEClipboard::IStyleTEClipboard(struct CApplication *aSupervisor,
					 Boolean hasWindow)
{
	CClipboard::IClipboard( aSupervisor, hasWindow);
}

/******************************************************************************
 MakeClipView
 
 	This method makes a panorama to display the current clipboard data. If
 	both 'TEXT' and 'styl' data are available it makes a CStyleText panorama,
 	otherwise it calls inherited to let CClipboard make the view.
 	
******************************************************************************/

CPanorama* CStyleTEClipboard::MakeClipView( long dataType, Handle dataHandle)
{
	Handle		styleHandle = NULL;
	CStyleText *text = NULL;
	Rect		margin;
	CPanorama	*panorama = NULL;
	
	TRY
	{
		if (dataType == 'TEXT')
		{		
			GetData( 'styl', &styleHandle);
			if (styleHandle)
			{
				text = new( CStyleText);
					
				text->IStyleText(itsScrollPane, this,
							1, 1, 0, 0, sizELASTIC, sizELASTIC, 1000);
				text->Specify( kNotEditable, kNotSelectable, kNotStylable);
				text->FitToEnclosure(TRUE, TRUE);
				text->Offset( 2, 2, FALSE);
				SetRect(&margin, 0, 0, -2, -2);
				text->ChangeSize( &margin, FALSE);
				
				MoveHHi( dataHandle);
				HLock( dataHandle);
				
				text->InsertWithStyles( *dataHandle, GetHandleSize( dataHandle),
							(StScrpHandle) styleHandle);
							
				ForgetHandle( dataHandle);  /* CStyleText makes its own copy. */
				ForgetHandle( styleHandle);		
				
				panorama = text;
			}
		}
		if (!panorama)
			panorama = inherited::MakeClipView( dataType, dataHandle);
	}
	CATCH
	{
		if (dataType == 'TEXT')
		{
			ForgetObject( text);
			ForgetHandle( dataHandle);
		}
	}
	ENDTRY;
	
	return panorama;
}
